# ca.tcl---automated CFD alignment

# Mon Aug 10 16:00:46 BST 1992, DB
#   changed adjustment policy: look for tardiest channel and set the other
#   channels' delays to align with it, rather than setting the promptest
#   channel's delay to 150ns. Got that?
# Wed Jul 15 19:48:37 BST 1992, DB
#   initial version

set debug 0
set frame CA
set frametitle "CFD Alignment"
set highest 75
set cloverstart 51
set lowchannel 1
set highchannel 1
set segment A
set caformat "
  (:command-frame $frame :label '$frametitle' :show-footer T
           :w 600 :h 260 :pin-in T
    (:panel CA-PANEL :x 0 :y 10 :w 600 :h 180 :layout V
       (:number CA-LOW :label 'Low channel:' :min 1 :max $highest :w 4 :valuex 110)
       (:number CA-HIGH :label 'High channel:' :min 1 :max $highest :w 4 :valuex 110)
       (:text CA-TIME :label 'Time (secs):' :w 8 :valuex 110)
       (:text CA-LEFT :label 'Zone left:' :w 8 :valuex 110)
       (:text CA-RIGHT :label 'Zone right:' :w 8 :valuex 110)
       (:message :label 'channel     cent      T(ns)       D(ns)   T+D(ns)'
             :x 250 :y 10 :bold T)
       (:list CA-POINTLIST :x 250 :y 30 :rows 6 :list-width 310 :exclusive T
           :strings ())
    )
    (:panel :x 0 :y 200 :w 600 :h 60
       (:button CA-CONTROL :label 'Start')
       (:button CA-CLEAR :label 'Clear list')
       (:button CA-ADJUST :label 'Adjust delays')
       (:button CA-HELP :x 500 :label 'Help')
     :next-row T
       (:button CA-REDISPLAY  :label 'Redisplay')
    ) :show T)"          

set state 0
set time 5
set zoneleft 2000
set zoneright 3000
set pointlist {{1 2608.99 2332.44 170 2502.44} {2 2697.23 2411.32 85 2496.32} {3 2651.41 2370.36 170 2540.36} {4 3000 2682 170 2852} {5 2643.12 2362.95 170 2532.95} {6 2637.36 2357.8 170 2527.8}}
set pointlist {}
set pointstrings {}
set selection -1
set spectrum ""
set latestfit {}
set phase 0
set timer ""
set chanbar 0
set timebar 0
set delay 0
set dplust 0
# the calibrations on Ge CFDs must be multiplied by the following factor
# to give true nanoseconds. DB Mon Aug 10 1992
set CFDfrigfactor 0.85
# and the trigger card TAC has been calibrated at the following
# nanoseconds per channel
set TACcal 0.894

proc first p {lindex $p 0}
proc second p {lindex $p 1}
proc third p {lindex $p 2}
proc fourth p {lindex $p 3}
proc fifth p {lindex $p 4}
proc sixth p {lindex $p 5}

proc dirnames l {
   foreach f $l {lappend ll [first $f]}
   return $ll
   }

proc format-points l {
   set ll {}
   foreach f $l {lappend ll [format "%3d%s %8.2f %8.2f %8.2f %8.2f"  \
      [first $f] [second $f] [third $f] [fourth $f] [fifth $f] [sixth $f]]}
   return $ll
   }

proc quote-list l {
   set ll ""
   foreach f $l {append ll " " "'$f'"}
   return $ll
   }

proc prefix-list {l p} {
   set ll {}
   foreach f $l {lappend ll $p$f}
   return $ll
   }

proc strip-dots l {
   set ll {}
   foreach n $l {if {$n!="." && $n!=".."} then {lappend ll $n}}
   return $ll
   }

proc clear-point {} {
   global chan chanerr calib caliberr
   set chan ""; set chanerr ""; set calib ""; set caliberr ""
   }

proc load-point n {
   global chan chanerr calib caliberr pointlist
   set point [lindex $pointlist $n]
   set chan [first $point]
   set chanerr [third $point]
   set calib [fourth $point]
   set caliberr [fifth $point]
   }

proc paint-pointlist {} {
   global pointlist pointstrings selection
   set pointstrings [format-points $pointlist]
   xv "(CA-POINTLIST :strings ([quote-list $pointstrings]))"
   if {$selection>=0} then {xv "(CA-POINTLIST :select $selection)"}
   }
                 
proc repaint {} {
   global state lowchannel highchannel time zoneleft zoneright
   if {$state==1} then {set label "Stop"} else {set label "Start"}
   xv "(CA-LOW :value $lowchannel)
       (CA-HIGH :value $highchannel)
       (CA-TIME :value '$time')
       (CA-LEFT :value '$zoneleft')
       (CA-RIGHT :value '$zoneright')
       (CA-CONTROL :label '$label')"
   }

proc set-footer f {
   global frame debug
   if {$debug} then {puts stderr $f}
   xv "($frame :left-footer '[quote $f]')"
   }

proc clear-footer {} {
   global frame
   xv "($frame :left-footer ' ')"
   }

proc list-take {as is} {
   set ll {}
   foreach e $is {lappend ll [lindex $as $e]}
   return $ll
   }

proc insert-log s {
   send sl insert [transparent $s]
   }

proc click-takefit {} {
   global latestfit chan chanerr spectrum
   if {$latestfit=={}} then \
      {set-footer "you haven't performed a peak fit!!!"; return}
   set fitlines [split $latestfit "\n"]
   if {[llength $fitlines]<3} then \
      {set-footer "sorry, can't process this fit---too few lines"; return}
   set titleline [first $fitlines]
   set titlewords [split $titleline " ,"]
   if {[first $titlewords]!="Background/Peak"} then \
      {set-footer "sorry, can't recognise this fit---not made by 'backpeak'"; return}
   set s [lindex $titlewords 4]
   if {$s!=""} then {set spectrum $s}
   set peakline [third $fitlines]
   set z [scan $peakline "%f (%f)" chan chanerr]
   if {$z!=2} then \
      {set-footer "sorry, can't process this fit---too few values"; return}
   repaint
   clear-footer
   }

proc check-point {} {
   global order units chan chanerr calib caliberr pointlist
   if {[scan $chan "%f" chan]!=1} then {return 1}
   if {[scan $chanerr "%f" chanerr]!=1} then {return 1}
   if {[scan $calib "%f" calib]!=1} then {return 1}
   if {[scan $caliberr "%f" caliberr]!=1} then {return 1}
   return 0
   }


proc split-at-equal s {split $s "="}

proc trim s {string trim $s}

proc map {p ss} {
   global $p
   set ll {}
   foreach s $ss {lappend ll [$p $s]}
   return $ll
   }

proc walk {p ss} {
   global $p
   foreach s $ss {$p $s}
   }

proc scan-float s {
   set f 0.0
   scan $s "%f" f
   return $f
   }

proc write-reg {r v} {
   global debug
   eg  "(register :name '/frontend/$r'  :value '$v')"
   if {$debug} then {puts stdout "(register :name '/frontend/$r'  :value '$v')"}
   }

proc write-regs {r v} {
   global debug
   eg "(register :pattern '/frontend/$r'  :value '$v')"
   if {$debug} then {puts stdout "(register :name '/frontend/$r'  :value '$v')"}
   }

proc read-reg {r} {
   eg "(register :name '/frontend/$r'  ?value)"
   }

proc read-regs {r} {
   eg "(register :pattern '/frontend/$r'  ?names-and-values)"
   }

proc parse-param p {
   global chanbar timebar peakarea backgroundarea
   if {[llength $p]<2} then {return}
   set value 0.0
   set label [trim [first $p]]
   set value [scan-float [second $p]]
   if {$label=="Centroid"} then {set chanbar $value}
   if {$label=="Energy"} then {set timebar $value}
   if {$label=="Background area"} then {set backgroundarea $value}
   if {$label=="Peak area"} then {set peakarea $value}
   }

# note frig factors calibrating TAC spectrum
# and CFD.Delay

proc fit-time-spectrum {} {
   global channel chanbar timebar pointlist zoneleft zoneright CFDfrigfactor
   global TACcal segment cloverstart
   set z [catch {exec cfdint -spectrum /hist/Trig.TAC \
                         -limit $zoneleft $zoneright} stuff]
   if {$z!=0} then {set-footer $stuff; return}      
   walk parse-param [map split-at-equal [split $stuff ",\n"]]
   set timebar [expr $chanbar*$TACcal]
   if {$channel >= $cloverstart} then {
      set delay [scan-float [read-reg Clover$channel.Ge$segment.CFD.Delay]]
   }  else {
      set delay [scan-float [read-reg Ge$channel.CFD.Delay]]
   }
   set delay [expr $delay*$CFDfrigfactor]
   set dplust [expr $delay+$timebar]
   if {$channel >= $cloverstart} then {
      lappend pointlist [list $channel $segment $chanbar $timebar $delay $dplust]
   }  else {
      lappend pointlist [list $channel " " $chanbar $timebar $delay $dplust]
   }
#   send sd show /hist/Trig.TAC
   }


proc trunc r {
   set i [string first "." $r]
   if {$i<0} then {return $r}
   incr i -1
   return [string range $r 0 $i]
   }

proc next-channel {} {
   global channel segment cloverstart highchannel debug
   if {$channel < $cloverstart} then {
      incr channel
      if {$channel==$cloverstart} then {
         set segment A
         set newchan $channel$segment
         set z [catch {eg "(register :name '/frontend/Clover$channel.Ge.LastCard' ?object)"} ]
      }  else {
         set newchan $channel
         set z [catch {eg "(register :name '/frontend/Ge$channel.LastCard' ?object)"} ]
      }
   } else {
      case $segment in {
         A { set segment B}
         B { set segment C}
         C { set segment D}
         D { set segment E}
         E { set segment A; incr channel}
         }
      set z [catch {eg "(register :name '/frontend/Clover$channel.Ge.LastCard' ?object)"} ]
      set newchan $channel$segment
   }
   if {$channel>$highchannel} then {set z 0}
   if {$debug} then {
      puts stdout "moved to channel $newchan, returning $z"
   }
   return $z
}

proc sequencer {} {
   global state phase timer lowchannel highchannel channel segment time pointlist cloverstart lastchannel
   case $phase in {
   0 {set channel $lowchannel
      if {$channel >= $cloverstart} then {
         set lastchannel Clover$channel.Ge$segment
      }  else {
         set lastchannel Ge$channel
      }
      set phase 1
      set timer [after 100 sequencer]}

   1 {if {$channel>$highchannel} then {
         set phase 99; set timer [after 100 sequencer]; return}
      set z [catch {
         write-reg Trig.VXI.Go 0
         write-regs *Ge\[?,??\].Enable 0
	 if {$channel >= $cloverstart} then {
            write-reg Clover$channel.Ge$segment.Enable 1
            set lastchannel Clover$channel.Ge$segment
         }  else {
            write-reg Ge$channel.Enable 1
            set lastchannel Ge$channel
         }
         eg "(spectrum :path '/hist' :name 'Trig.TAC' !zero)"
         write-reg Trig.VXI.Go 1
         } m]
      if {$z!=0} then {set state 0; set-footer \
         "unable to start accumulation: $m"; return}
      if {$channel >= $cloverstart} then {
         set-footer "accumulating channel Clover$channel.Ge$segment..."
      } else {
         set-footer "accumulating channel Ge$channel..."
      }
      set phase 2; set timer [after [expr $time*1000] sequencer]}

   2 {
      if {$channel >= $cloverstart} then {
         set-footer "fitting TAC spectrum for channel Clover$channel.Ge$segment..."
      } else {
         set-footer "fitting TAC spectrum for channel Ge$channel..."
      }
      fit-time-spectrum
      paint-pointlist
      set ok 1
      while {$ok!=0} {set ok [next-channel]}
      set phase 1; set timer [after 100 sequencer]}

  99 {   
      set-footer "completed TAC measurements (only $lastchannel enabled)"
      set state 0
      repaint}      }      
   }

proc click-clear {} {
   global selection pointlist
   set pointlist {}
   set selection -1
   paint-pointlist
   set-footer "cleared list of TAC measurements"
   }

proc click-adjust {} {
   global pointlist CFDfrigfactor cloverstart
# find the minimum dplust and its d
   set mindplust 9999
   foreach p $pointlist {
      if {[sixth $p]<$mindplust} then \
         {set mindplust [sixth $p]; set mind [fifth $p]}
      }
# adjust the delays
# note the recalibration factor
   foreach p $pointlist {
      set d [expr ([sixth $p]-$mindplust+$mind)/$CFDfrigfactor]
      set c [first $p]
      set s [second $p]
      if {$c>=$cloverstart} then {
         set-footer "setting Clover$c.Ge$s.CFD.Delay to ${d}ns..."
         write-reg Clover$c.Ge$s.CFD.Delay ${d}ns
      } else {
         set-footer "setting Ge$c.CFD.Delay to ${d}ns..."
         write-reg Ge$c.CFD.Delay ${d}ns
      }
   }
   set-footer "adjusted CFD delays"
   }

proc click-control {} {
   global state phase timer
   clear-footer
   if {$state==0} then {
      set state 1
      set phase 0; set timer [after 100 sequencer]} \
   else {
      set state 0
      if {$timer!=""} then {
         kill-after $timer
         set-footer "accumulation sequence interrupted"
         }
      }
   repaint
   }

proc click-help {} {
   send mb goto [transparent "CFD Alignment"]
   }

proc click-pointlist {c s} {
   global pointstrings selection
   case $c in {
     :list-select 
        {set selection [lsearch $pointstrings $s]; load-point $selection}
     :list-deselect 
        {set selection -1}
     }          
   repaint
   clear-footer
   }

proc click-low v {
   global lowchannel highchannel highest
   set lowchannel $v
   if {$lowchannel<1} then {set lowchannel 1}
   if {$lowchannel>$highest} then {set lowchannel $highest}
   if {$highchannel<$lowchannel} then {set highchannel $lowchannel}
   repaint
   }

proc click-high v {
   global lowchannel highchannel highest
   set highchannel $v
   if {$highchannel<1} then {set highchannel 1}
   if {$highchannel>$highest} then {set highchannel $highest}
   if {$lowchannel>$highchannel} then {set lowchannel $highchannel}
   repaint
   }

proc click-time v {
   global debug time
   if {$debug} then {puts stdout "$v"}
   set z [scan $v "%f" zz]
   if {$z==0} then {set-footer "$v is not an acceptable number"; return}
   set time $v
   clear-footer
   }

proc click-left v {
   global debug zoneleft
   if {$debug} then {puts stdout "$v"}
   set z [scan $v "%f" zz]
   if {$z==0} then {set-footer "$v is not an acceptable number"; return}
   set zoneleft $v
   clear-footer
   }

proc click-right v {
   global debug zoneright
   if {$debug} then {puts stdout "$v"}
   set z [scan $v "%f" zz]
   if {$z==0} then {set-footer "$v is not an acceptable number"; return}
   set zoneright $v
   clear-footer
   }

proc click-quit {} {xv "(CA :delete)"}

proc click-redisplay {} {
   paint-pointlist
   repaint
   clear-footer
   }

proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {remove-eg-error-report self; return}
   if {$c==":quit"} then {click-quit; return}
   case $w in {
      CA-CONTROL {click-control}
      CA-CLEAR {click-clear}
      CA-ADJUST {click-adjust}
      CA-POINTLIST {click-pointlist $c $v}
      CA-HIGH {click-high $v}
      CA-LOW {click-low $v}
      CA-TIME {click-time $v}
      CA-LEFT {click-left $v}
      CA-RIGHT {click-right $v}
      CA-HELP {click-help}
      CA-REDISPLAY {click-redisplay}
      default    {puts stdout $arg}
      }
   }

proc self op {
   global frame frametitle
   case $op in {
      frame-name {return $frame}
      frame-title {return $frametitle}
      }
   }

proc start args {
   global caformat
   xv $caformat
   paint-pointlist
   repaint
   }

proc finish args {click-quit}

register-interest ca
