# cluster.tcl---cluster card control

# created 26-6-95 VP  from clover 

set debug 0
set started 0

set frame CLUSTER
set frametitle "Cluster Card Setup"
set icontitle  "Cluster"

set chanlist {???}
set chanindex 0
set Ge_seglist {A B C D E F G H}
set Ge_spareseg [last $Ge_seglist]
set BGO_seglist {A B C D E F G H I J K L M N O P Q R S}
set BGO_spareseg [last $BGO_seglist]
set segindex -1

set defaultchanlist {0 1 2 3 4 5 6 7}

set cardtype ClusterGe
set chantype Ge
set seglist $Ge_seglist


#    The array "table" which is indexed by the "widget name" contains all the information needed
#    for the register/display item
#
#    Each element of "table" is a list
#
#       0.    name of global parameter used to hold current value of the register
#       1.    procedure to be used to convert the widget value into parameter value
#       2.    procedure to be used to convert the parameter value into a widget value
#       3.    name of the register
#       4.    procedure to be used to convert the register value into parameter value
#       5.    procedure to be used to convert the parameter value into a register value
#       6.    name of unit used for regsiter access
#       7.    widget type to be used for "by parameter" displays
#       8-10  parameters for the procedure creating "by parameter" displays
#              eg  if (7) is :slider then 8 is min value, 9 is max value and 10 is unit
#       11    for startup only - initial value of the parameter (1)
#       12    for startup only - inserted for this widget into the table "parameternames"

#   These table entries relate to the main window


set table(ALLCHANS) {allchans int-widget->param param->int-widget nil nil nil nil nil nil nil nil 0 nil}
set table(ALLSEGS)  {allsegs  int-widget->param param->int-widget nil nil nil nil nil nil nil nil 0 nil}

foreach i {A B C D E F G H} {
  set table(ENABLE-$i) "enable[string tolower $i] int-widget->param param->int-widget
    \"Ge$i.Enable\" int-reg->param param->int-reg nil :checkbox nil nil nil 1 \"Ge$i.Enable\" "
}

foreach i {A B C D E F G H I J K L M N O P Q R S Sum} {
  set table(ENABLE-BGO-[string toupper $i]) "enablebgo[string tolower $i] int-widget->param param->int-widget
    \"BGO$i.Enable\" int-reg->param param->int-reg nil :checkbox nil nil nil 1 \"BGO$i.Enable\""
}

set table(ENABLE-BGO-PATT-L) {enablebgopattl int-widget->param param->int-widget "BGOPattern.Low.ROEnable" 
   int-reg->param param->int-reg nil :checkbox nil nil nil 1 nil}

set table(ENABLE-BGO-PATT-H) {enablebgopatth int-widget->param param->int-widget "BGOPattern.High.ROEnable" 
   int-reg->param param->int-reg nil :checkbox nil nil nil 1 nil}


set table(SPARE-GE) {sparege spare-widget->param param->int-widget "GeH.4MeV.Addr" 
   spare-reg->param param->spare-reg nil nil nil nil nil 1 nil}

set table(SPARE-BGO) {sparebgo spare-widget->param param->int-widget "BGOS.Energy.Addr" 
   spare-reg->param param->spare-reg nil nil nil nil nil 1 nil}


loadsource common.vxi

#proc EG {args} {puts stdout "EG: ($args)" }

proc selectcard {type} {
   global cardtype chantype Ge_seglist BGO_seglist
   set cardtype Cluster$type
   set chantype $type
   set seglist [set [set type]_seglist]
}

proc click-redisplay {} {
   global cardtype chantype
   global chanlist chanindex unshown
   global allsegments

   if {![:query_Mapped .]} then {return}

#    first do Ge part

   selectcard Ge
#   update-pairs [read-regs [lindex $chanlist $chanindex] $allsegments *]
   Redisplay

#    then do BGO part

   selectcard BGO
#   update-pairs [read-regs [lindex $chanlist $chanindex] * *]
   Redisplay

   set unshown 0
}   


proc click-spare-ge v {
	global debug cardtype chantype chanlist chanindex Ge_spareseg
        global segindex

        selectcard Ge

	set adcs {20MeV 4MeV FT}
	set seg [lindex {H A B C D E F G} $v]
	set chan [lindex $chanlist $chanindex]

        set segindex $v

	if {$v!=0} then {
		set val [read-reg $chan $seg Enable]
		write-reg $chan $Ge_spareseg Enable $val 0
		write-reg $chan $seg         Enable    0 0
	}
	foreach adc $adcs {
		write-reg $chan $Ge_spareseg $adc.Addr $cardtype$chan.$chantype$seg.$adc 0
	}

        set segindex -1

	click-redisplay
}

proc click-spare-bgo v {
	global debug cardtype chantype chanlist chanindex BGO_spareseg
        global segindex

        selectcard BGO 

	set adcs {Energy TAC}
	set seg [lindex {S A B C D E F G H I J K L M N O P Q R} $v]
	set chan [lindex $chanlist $chanindex]

        set segindex $v

	if {$v!=0} then {
		set val [read-reg $chan $seg Enable]
		write-reg $chan $BGO_spareseg Enable $val 0
		write-reg $chan $seg          Enable    0 0
	}
	foreach adc $adcs {
		write-reg $chan $BGO_spareseg $adc.Addr $cardtype$chan.$chantype$seg.$adc 0
	}

        set segindex -1

	click-redisplay
}

proc click-ge seg {
	global Ge_seglist
	inform clusterge start
	inform clusterge click-segment [lsearch $Ge_seglist $seg]
}

proc click-bgo seg {
        global BGO_seglist
        inform clusterbgo start
        inform clusterbgo click-segment [lsearch $BGO_seglist $seg]
}

proc click-enablebgo-global {v} {
   global chanlist chanindex
   set chan [lindex $chanlist $chanindex]
   write-common-reg $chan BGO?.Enable $v 0
   click-redisplay
}


proc click arg {
   global debug env cardtype chantype
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {return [click-quit $f]}
   set cardtype Cluster*
   set chantype *
   switch -glob $w  {
      REDISPLAY   {click-redisplay}
      GE-MENU     {click-ge $v}
      GE          { }
      BGO-MENU    {click-bgo $v}
      BGO         { }
      SPARE-GE    {click-spare-ge $v}
      SPARE-BGO   {click-spare-bgo $v}
      ENABLE-BGO-ALL  {
          selectcard BGO
          click-enablebgo-global 1
          :modify $w :value 0

      }
      ENABLE-BGO-NONE  {
          selectcard BGO
          click-enablebgo-global 0
          :modify $w :value 0
      }
      ENABLE-BGO-*  {
          selectcard BGO
          click-on $w $v
      }
      ENABLE-*  {
          selectcard Ge
          click-on $w $v
      }
      HELP        {click-help}
      CHANNEL     {
                    selectcard Ge
                    click-channel $v
                    inform clusterge click-channel $v
                    inform clusterbgo click-channel $v
                  }

      ALLCHANS  {click-whichchans $v}

      default     {click-on $w $v}
   }
}


proc make-frame1 {} {
  global env chanlist Ge_seglist BGO_seglist

  :frame "[self frame-name]" :label "[self frame-title]" :show-footer T   \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]" :resize F

    :panel PANEL :layout T :padx 5 :pady 5
      :layout H; :row-gap 10

      :choice-stack CHANNEL :label "Channel:" :valuex 15c :strings $chanlist
			
      :checkbox ALLCHANS :label "Act on all Channels:" :valuex 25c
      :row-gap 20; :next-row T; :row-gap 10
      :choice ENABLE-A   :exclusive F :strings "A" :label "Enable Ge:" :w 3 :valuex 15c
      foreach i {B C D E F G H} {
        :choice ENABLE-$i :exclusive F :strings "$i" :w 3
      }
      :next-row T	
      :choice ENABLE-BGO-A   :exclusive F :strings "A" :label "Enable BGO:" :w 3 :valuex 15c
      foreach i {B C D E F G H} {
        :choice ENABLE-BGO-$i :exclusive F :strings "$i" :w 3
      }
      :row-gap 5; :next-row T	
      :choice ENABLE-BGO-I   :exclusive F :strings "I" :w 3 :valuex 15c
      foreach i {J K L M N O P} {
        :choice ENABLE-BGO-$i :exclusive F :strings "$i" :w 3
      }
      :next-row T	
      :choice ENABLE-BGO-Q   :exclusive F :strings "Q"  :w 3 :valuex 15c
      foreach i {R S} {
        :choice ENABLE-BGO-$i :exclusive F :strings "$i" :w 3
      }
      :choice ENABLE-BGO-ALL   :exclusive F :strings "All" :w 3 :valuex 15c
      :choice ENABLE-BGO-NONE   :exclusive F :strings "None" :w 4
      :row-gap 10; :next-row T
      :col-gap 10
      :message  M1  :label "Enable BGO"
      :checkbox ENABLE-BGO-SUM    :label  "Sum Energy"
      :checkbox ENABLE-BGO-PATT-L :label  "Pattern Low"
      :checkbox ENABLE-BGO-PATT-H :label  "Pattern High"
      :next-row T
      :choice-stack SPARE-GE :label "Spare Ge Channel:" :valuex 30c        \
	    :strings [list "Unused" "Position A" "Position B" "Position C" \
			       "Position D" "Position E" "Position F"      \
                               "Position G"]
      :next-row T
      :choice-stack SPARE-BGO :label "Spare BGO Channel:" :valuex 30c       \
	    :strings [list "Unused" "Position A" "Position B" "Position C"  \
				    "Position D" "Position E" "Position F"  \
                                    "Position G" "Position H" "Position I"  \
                                    "Position J" "Position K" "Position L"  \
                                    "Position M" "Position N" "Position O"  \
                                    "Position P" "Position Q" "Position R"]

    :panel BUTTONS :layout T :padx 5 :pady 5
     :layout H; :row-gap 10
      :menubutton GE-MENU  :label "GE Channel"   :strings $Ge_seglist
      :menubutton BGO-MENU :label "BGO Channel"  :strings $BGO_seglist
      :next-row T
      :button REDISPLAY :label "Redisplay"
      :hskip 80
      :button HELP      :label "Help..."

    :show T
}
