#    hist task code overlay for generic type "group"

set lowchannel 0
set highchannel 0
set maxchannel 255
set qualifier 1

set lowitem 0
set highitem 0
set maxitem 63

set lowrange 1
set highrange 1
set maxrange 65536
set rangeofitem 16384

set itemmenu  {0-0 0-1 0-2 1 2 3 4 8 16 32 Reset}
set rangemenu {128 256 512 1024 2K 4K 8K 16K 32K 64K}
set qnames    {q0 q1 q2 q3}
set qualnames {0 1 2 3}


proc make-overlaypanel {} {
   global maxchannel maxitem rangemenu itemmenu qnames

    :layout V; :row-gap 10; :col-gap 0; :row-gap 10

      :message DET-M1  :label "generic detector overlay applied"

      :number DET-LOW  :label "Low group:"  :min 0 :max $maxchannel :w 5 :valuex 15c
      :number DET-HIGH :label "High group:" :min 0 :max $maxchannel :w 5 :valuex 15c

      :choice DET-QUAL :label "Qualifier"  :valuex 15c :exclusive F :strings $qnames   :value 1

      :number DET-LOWI  :label "Low item:"  :min 0 :max $maxitem :w 5 :valuex 15c
      :number DET-HIGHI :label "High item:" :min 0 :max $maxitem :w 5 :valuex 15c

      :hskip 40
      :menubutton DET-ITEM-MENU :label "Items" :strings $itemmenu

      :menubutton DET-RANGE-MENU :label "Channels:" :strings $rangemenu
      :layout H
      :number DET-RANGE :label "Range(channels):" :min 0 :max 65536 :w 5
}


#    this proc handles events from widgets created in the proc make-overlaypanel
#    The widgets DET-LOW; DET-HIGH and DET-QUAL are handled in the main click procedure
#    and are not required here

proc click-overlay {w v} {

   switch -glob $w {
      DET-LOWI           {click-low-item $v}
      DET-HIGHI          {click-high-item $v}
      DET-ITEM-MENU      {click-item-menu $v}
      DET-RANGE-MENU     {click-range-menu $v}
      DET-RANGE          {click-range $v}
      default      {puts stdout $arg}
   }
}

proc click-low-item v {
   global lowitem highitem maxitem
   set lowitem $v
   if {$lowitem<0} then {set lowitem 0}
   if {$lowitem>$maxitem} then {set lowitem $maxitem}
   if {$highitem<$lowitem} then {set highitem $lowitem}
   repaint
}

proc click-high-item v {
   global lowitem highitem maxitem
   set highitem $v
   if {$highitem<0} then {set highitem 0}
   if {$highitem>$maxitem} then {set highitem $maxitem}
   if {$lowitem>$highitem} then {set lowitem $highitem}
   repaint
}

proc click-item-menu v {
    global lowitem highitem maxitem
    switch $v  {
            Reset {
                 set highitem 0
                 set lowitem 0
            }
            0-0 {
                 set highitem 0
                 set lowitem 0
            }
            0-1 {
                 set highitem 1
                 set lowitem 0
            }
            0-2 {
                 set highitem 2
                 set lowitem 0
            }
            default {
                 set z [scan $v "%d" n]
                 if {$z==1} then {
                     if {[expr $lowitem+$n-1]==$highitem} then \
                        {incr lowitem $n; incr highitem $n} \
                     else {set highitem [expr $lowitem+$n-1]}
                 }
            }
    }
    if {$lowitem<0} then {set lowitem 0}
    if {$lowitem>$maxitem} then {set lowitem $maxitem}
    if {$highitem<$lowitem} then {set highitem $lowitem}
    repaint
}

proc click-range v {
   global lowrange highrange maxrange rangeofitem
   set rangeofitem $v
   if {$rangeofitem<1} then {set rangeofitem 1}
   if {$rangeofitem>$maxrange} then {set rangeofitem $maxrange}
   repaint
}

proc click-range-menu v {
    switch $v {
           2K { click-range 2048}
           4K { click-range 4096}
           8K { click-range 8192}
          16K { click-range 16384}
          32K { click-range 32768}
          64K { click-range 65536}
          default {
               set z [scan $v "%d" n]
               if {$z==1} then {click-range $n}
          }
     }
}

proc repaint_overlay {} {
   global lowitem highitem rangeofitem

   :modify DET-LOWI  :value $lowitem
   :modify DET-HIGHI :value $highitem
   :modify DET-RANGE :value $rangeofitem
}



proc get-spectrum-list {} {
        global spectrumlist
        global qualifier qualnames
        global lowchannel highchannel lowitem highitem

        if {$spectrumlist != {}} then {return $spectrumlist};  # return names selected from spectrum list


        set qlist [list-take $qualnames [bits-in $qualifier]]
        set list {}

        for {set i $lowchannel} {$i<=$highchannel} {incr i} {
# append spectrum names
           for {set j $lowitem} {$j<=$highitem} {incr j} {
              foreach q $qlist {lappend list Group$i.Item$j.q$q}
           }
        }
       return $list
}

proc range-of n {
  global rangeofitem
  return $rangeofitem
}
