# rdout.tcl    used by all modules having RdOut readout

#  12-08-96   VP   first version 

#    The array "table" which is indexed by the "widget name" contains all the information needed
#    for the register/display item
#
#    Each element of "table" is a list
#
#       0.    name of global parameter used to hold current value of the register
#       1.    procedure to be used to convert the widget value into parameter value
#       2.    procedure to be used to convert the parameter value into a widget value
#       3.    name of the register
#       4.    procedure to be used to convert the register value into parameter value
#       5.    procedure to be used to convert the parameter value into a register value
#       6.    name of unit used for register access
#       7.    widget type to be used for "by parameter" displays
#       8-10  parameters for the procedure creating "by parameter" displays
#              eg  if (7) is :slider then 8 is min value, 9 is max value and 10 is unit
#       11    for startup only - initial value of the parameter (1)
#       12    for startup only - inserted for this widget into the table "parameternames"


set table(RDOUTEVNUMADDR)    {rdoutevnumaddr     int-widget->param param->hex-widget "RdOut.EvNum.Addr"
                     int-reg->param param->int-reg "" nil nil nil nil 0 nil}
set table(RDOUTEVNUMCOUNT)    {rdoutevnumcount    int-widget->param param->hex-widget "RdOut.EvNum.Count"
                     int-reg->param param->int-reg "" nil nil nil nil 0 nil}

set table(RDOUTROCRREADOUTMODE)     {rdoutreadoutmode     logic-widget->param param->logic-widget "RdOut.ROCR.ReadOutMode"
                     int-reg->param param->int-reg "" nil nil nil nil 0 nil}
set table(RDOUTROCREVNUMDATAENABLE) {rdoutevnumdataenable logic-widget->param param->logic-widget "RdOut.ROCR.EvNumData.Enable"
                     int-reg->param param->int-reg "" nil nil nil nil 0 nil}
set table(RDOUTROCREVNUMQBITSENABLE) {rdoutevnumqbitsenable logic-widget->param param->logic-widget "RdOut.ROCR.EvNumQbits.Enable"
                     int-reg->param param->int-reg "" nil nil nil nil 0 nil}
set table(RDOUTROCRPIPELINEENABLE) {rdoutpipelineenable logic-widget->param param->logic-widget "RdOut.ROCR.PipeLine.Enable"
                     int-reg->param param->int-reg "" nil nil nil nil 0 nil}
set table(RDOUTROCREVFAULT1) {rdoutevfault1 logic-widget->param param->logic-widget "RdOut.ROCR.EvFault1"
                     int-reg->param param->int-reg "" nil nil nil nil 0 nil}
set table(RDOUTROCREVFAULT2) {rdoutevfault2 logic-widget->param param->logic-widget "RdOut.ROCR.EvFault2"
                     int-reg->param param->int-reg "" nil nil nil nil 0 nil}
set table(RDOUTROCRFIFO1EMPTY) {rdoutfifo1empty logic-widget->param param->logic-widget "RdOut.ROCR.FIFO1empty"
                     int-reg->param param->int-reg "" nil nil nil nil 0 nil}
set table(RDOUTROCRFIFO2EMPTY) {rdoutfifo2empty logic-widget->param param->logic-widget "RdOut.ROCR.FIFO2empty"
                     int-reg->param param->int-reg "" nil nil nil nil 0 nil}
set table(RDOUTROCRFIFO3EMPTY) {rdoutfifo3empty logic-widget->param param->logic-widget "RdOut.ROCR.FIFO3empty"
                     int-reg->param param->int-reg "" nil nil nil nil 0 nil}
set table(RDOUTROCRFIFO1FULL) {rdoutfifo1full logic-widget->param param->logic-widget "RdOut.ROCR.FIFO1full"
                     int-reg->param param->int-reg "" nil nil nil nil 0 nil}
set table(RDOUTROCRFIFO2FULL) {rdoutfifo2full logic-widget->param param->logic-widget "RdOut.ROCR.FIFO2full"
                     int-reg->param param->int-reg "" nil nil nil nil 0 nil}
set table(RDOUTROCRFIFO3FULL) {rdoutfifo3full logic-widget->param param->logic-widget "RdOut.ROCR.FIFO3full"
                     int-reg->param param->int-reg "" nil nil nil nil 0 nil}


proc click-on-RdOut {w v} {click-on $w $v}

proc click-RdOut {} {
#  display RdOut sub frame
   global rdoutvisible rdoutframe
   if {$rdoutvisible==0} then {make-RdOut} else {open-frame $rdoutframe}
   click-redisplay
} 

set rdoutframe [self frame-name].rdout
set rdoutvisible 0

proc make-RdOut {} {
  global env rdoutvisible rdoutframe

set col1 40c

  :frame $rdoutframe :label "RdOut" :show-footer T :resize F :transient T 

  :panel PANEL1 :layout T :padx 5 :pady 5
    :layout V; :row-gap 5; :col-gap 5

      :text     RDOUTEVNUMADDR     :label "Event Number Address:"  :valuex $col1 :w 6  :deaf T
      :text     RDOUTEVNUMCOUNT    :label "Event Number Counter:"  :valuex $col1 :w 6

      :vskip 10
      :message M1 :label "Readout Control Register" :bold T
      :choice RDOUTROCRREADOUTMODE         :label "Read Out Mode" :valuex 25c  \
          :strings [list "Common Dead Time" "Parallel"]
      :checkbox RDOUTROCREVNUMDATAENABLE   :label "Event Number Readout Enable:"          :valuex $col1
      :checkbox RDOUTROCREVNUMQBITSENABLE  :label "Use Event Number Qbits:"               :valuex $col1
      :checkbox RDOUTROCRPIPELINENABLE     :label "Pipeline readout enable:"              :valuex $col1
      :checkbox RDOUTROCREVFAULT1          :label "Event Fault type 1:"                   :valuex $col1
      :checkbox RDOUTROCREVFAULT2          :label "Event Fault type 2:"                   :valuex $col1
      :checkbox RDOUTROCRFIFO1EMPTY        :label "FIFO #1 empty:"                        :valuex $col1
      :checkbox RDOUTROCRFIFO2EMPTY        :label "FIFO #2 empty:"                        :valuex $col1
      :checkbox RDOUTROCRFIFO3EMPTY        :label "FIFO #3 empty:"                        :valuex $col1
      :checkbox RDOUTROCRFIFO1FULL         :label "FIFO #1 full:"                         :valuex $col1
      :checkbox RDOUTROCRFIFO2FULL         :label "FIFO #2 full:"                         :valuex $col1
      :checkbox RDOUTROCRFIFO3FULL         :label "FIFO #3 full:"                         :valuex $col1

  :panel BUTTONS :layout B :padx 5 :after FOOTER
      :layout H
      :button RDOUTREDISPLAY  :label Redisplay

  :show T
  set rdoutvisible 1
}

