set debug 0
set started 0

set frame REPORTER
set frametitle "Reporting a fault"
set icontitle Reporter


set currentpage 0
set lastpage 9

set directory [file join $env(MIDASEXPBASE) Reports]
set maillist "pucknell@dl.ac.uk,lazarus@dl.ac.uk,maron@lnl.infn.it"

proc null args {}

proc make_page {n} {
   global env _variable currentpage
   upvar #0 page[set n]text text

  set currentpage $n

  :frame "[self frame-name]" :label "[self frame-title]" :show-footer T  :resize F :toplevel T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

    :panel PANEL1 :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 10
      :colour local text background $env(tkframebackground)
      :colour local text foreground black
      :galley GALLEY  :scroll N :rows 10 :cols 80 :deaf T
      get_text GALLEY
      set _variable(emptylist) $_variable(galley)
      :modify GALLEY :strings $text
}

proc make_buttons {} {

    :panel BUTTONS :layout B :padx 10 :pady 10 :after FOOTER
    :layout H; :col-gap 10; :row-gap 10
      :button CANCEL   :label "Cancel"  :command cancel
      :button BACK     :label "<Back"   :command goto_previouspage
      :button NEXT     :label "Next>"   :command goto_nextpage
      :button FINISH   :label "Finish"  :command goto_finish  :inactive T

   centre-frame
   :show T
}

proc get_text {w} {
   global _variable
   set v [string tolower $w]
   set _variable($v) "[[:path $w].text get 1.0 end]"
}

proc value {w} {
   global _variable
   set v [string tolower $w]
   if {[info exists _variable($v)]} then {return $_variable($v)} else {return ""}
}

proc goto_nextpage {} {
   global currentpage lastpage
   switch $currentpage {
     4  {get_text DESCRIPTION}
     6  {get_text DETECTOR}
     7  {get_text HISTS}
     8  {get_text REPEATABLE}
     default {}
   }
   if {$currentpage == $lastpage} then {goto_finish; return}
   incr currentpage
   make_page[set currentpage]
   if {$currentpage == $lastpage} then {
     :modify NEXT :inactive T
     :modify FINISH :inactive F
   }
   if {$currentpage == 1} then {
     :modify BACK :inactive T
   }
}

proc goto_previouspage {} {
   global currentpage lastpage
   incr currentpage -1
   if {$currentpage == 0} then {set currentpage 1}
   make_page[set currentpage]
}



set page1text "Welcome to the Report Generator\n\nYou will be asked a number of questions concerning the fault you wish to report. The more information you can give the more likely it is that the fault can be fixed quickly. However please try to ensure that the information you give is correct and accurate.\n\nThank you."

proc make_page1 {} {
   make_page 1
   make_buttons 
}


set page2text "Please give the date and time when you first observed the fault.\n\nThe system keeps log files recording history information concerning the hardware and we may be able to find useful information from these files if we know when the fault started."

proc make_page2 {} {
  make_page 2

  :panel PANEL2 :layout T :padx 10 :pady 10
    :layout V; :row-gap 5
   :text DATE  :valuex 8c :w 32  :command null :value [value DATE]
   :text TIME  :valuex 8c :w 32  :command null :value [value TIME]

  make_buttons
}


set page3text "Please give us information so that we can contact you.\n\nIt may be necessary to obtain more information or to discuss the fault with you so it would be useful if we know who you are and have a way of contacting you. Also we can keep you informed of progress and let you know when the fault is corrected." 

proc make_page3 {} {
  make_page 3

  :panel PANEL2 :layout T :padx 10 :pady 10
    :layout V; :row-gap 5
   :text NAME    :label "your name"              :valuex 20c :w 32  :command null :value [value NAME]
   :text EMAIL   :label "your e-mail address"    :valuex 20c :w 32  :command null :value [value EMAIL]
   :text FAX     :label "your fax number"        :valuex 20c :w 32  :command null :value [value FAX]
   :text PHONE   :label "your telephone number"  :valuex 20c :w 32  :command null :value [value PHONE]

  make_buttons
}


set page4text "Please tell us the fault.\n\nGive as much information as you can but try to ensure that the information is accurate. A lot of time can be wasted tracing faults based on incorrect reports.\n\nIf the fault is related to a specific electronics module it is important that we know the name of the module as you access it from the graphics screen.\nFor example -  ClusterGe channel 5 segment C."

proc make_page4 {} {
  global env _variable
  make_page 4

  :panel PANEL2 :layout T :padx 10 :pady 10
    :layout V; :row-gap 5
      :colour local text background $env(tktextbackground)
      :colour local text foreground $env(tktextforeground)
      :galley DESCRIPTION  :rows 6 :cols 75 :strings "$_variable(description)"

  make_buttons
}


set page5text "We now would like details of the configuration in use when the fault occurred.\n\nIf you are reporting a fault with the VXI electronics the most useful information is the name of the VXI parameter setup file in use. If you are unsure of this information start the \"Restoring VXI Module Setups\" window and give us the directory at the very top of the window and the save directory from the list window which you are currently using."

proc make_page5 {} {
  make_page 5

  :panel PANEL2 :layout T :padx 10 :pady 10
    :layout V; :row-gap 5
   :text SETUP    :label "name of the VXI setup in use"   :valuex 24c :w 55  :command null :value [value SETUP]

  make_buttons
}


set page6text "If you are just using the basic Euroball array (that is you are NOT using any ancillary detectors) please click on the \"Next\" button to proceed.\n\nHowever if you are using an ancillary detector please gives details of the detector in use and how you are connecting it to Euroball."

proc make_page6 {} {
  global env _variable
  make_page 6

  :panel PANEL2 :layout T :padx 10 :pady 10
    :layout V; :row-gap 5
      :colour local text background $env(tktextbackground)
      :colour local text foreground $env(tktextforeground)
      :galley DETECTOR  :rows 6 :cols 75 :strings "$_variable(detector)"

  make_buttons
}


set page7text "Please give the filenames of data saved as histograms which demonstrate the fault.\n\nEspecially if the fault is detected as a result of data seen in a histogram it will help a lot if you save the histogram(s) and let us know where they have been saved.\nIf necessary use the \"Save\" button in the \"Hardware Histogrammer\" window to start the \"Spectrum Copy and Online Histogram Save\" window. Remember we will need to know the ID you are currently logged on as and the parent directory and save directory used. These are the 2 directory names on the right hand size of the window"

proc make_page7 {} {
  global env _variable
  make_page 7

  :panel PANEL2 :layout T :padx 10 :pady 10
    :layout V; :row-gap 5
      :colour local text background $env(tktextbackground)
      :colour local text foreground $env(tktextforeground)
      :galley HISTS  :rows 6 :cols 75 :strings "$_variable(hists)"

  make_buttons
}


set page8text "Can the fault be easily re-created? If it is totally random and you cannot see any event or sequence of actions which cause it then click on the \"Next\" button to proceed.\n\nHowever if the fault does appear to be repeatable what actions cause the fault to occur.\n\nIf the fault is intermittent how often does it occur? Once a minute? Once an hour? Only one a day? You may leave this box empty if the fault does not appear to occur at fixed intervals."

proc make_page8 {} {
  global env _variable
  make_page 8

  :panel PANEL2 :layout T :padx 10 :pady 10
    :layout V; :row-gap 5
      :colour local text background $env(tktextbackground)
      :colour local text foreground $env(tktextforeground)
      :galley REPEATABLE  :rows 6 :cols 75 :strings "$_variable(repeatable)"
      :text FREQUENCY  :label "frequency of the fault"   :valuex 20c :w 50  :command null :value [value FREQUENCY]

  make_buttons
}


set page9text "Finally we would like to know how serious you rate the fault.\n\nPlease be realistic. It does not help us if all faults are rated as critical since we will just apply our judgement as to what to do."
set page9choice [list \
           "Fault is critical - it prevents the experiment from continuing" \
           "Fault is serious - I would like a fix during the experiment if possible" \
           "The experiment can continue but the fault should be fixed before the next experiment" \
           "The fault is not serious and a fix can wait until a suitable planned shutdown"]

proc make_page9 {} {
  global page9choice
  make_page 9

  :panel PANEL2 :layout T :padx 10 :pady 10
    :layout V; :row-gap 5
      :choice SEVERITY :layout V :w 80 :value -1 :strings $page9choice

  make_buttons
}

proc click-severity args {
   global _variable page9choice
   set _variable(severity) "[lindex $page9choice $args]"
   goto_nextpage
}


set pageftext "Thank you. You have supplied all the information required at present\n\nPlease check the Fault Report produced from your answers. If all is correct then click on the \"Complete\" button to submit the fault report. If you wish to make changes then click on the \"Change\" button."

proc goto_finish {} {
  make_page f

  :panel PANEL2 :layout T :padx 10 :pady 10
    :layout V; :row-gap 5
      :galley REPORT  :rows 20 :cols 75 

  :panel BUTTONS :layout B :padx 10 :pady 10 :after FOOTER
    :layout H; :col-gap 10; :row-gap 10
      :button CANCEL     :label "Cancel"  :command cancel
      :button CHANGE     :label "Change"  :command make_page2
      :button COMPLETE   :label "Complete"  :command complete

   show_report

   centre-frame
   :show T
}

proc show_report {} {
  global _variable
  :modify REPORT :empty T
  :modify REPORT :strings "Euroball Fault Report generated [date]"
#    page 2
  :modify REPORT :strings "\n\n1)Date and time that the fault occurred.\n>Date: $_variable(date)\n>Time: $_variable(time)\n\n"
#    page 3
  :modify REPORT :strings "\n\n2)Person reporting the fault.\n>name: $_variable(name)\n>e-mail address: $_variable(email)\n>fax: $_variable(fax)\n>telephone: $_variable(phone)"
#    page 4
  :modify REPORT :strings "\n\n3)Description of the fault.\n>$_variable(description)"
#    page 5
  :modify REPORT :strings "\n\n4)Details of the configuration in use when the fault occurs.\n>Name of VXI setup in use: $_variable(setup)"
#    page 6
  if {$_variable(detector) == $_variable(emptylist)} then {
    :modify REPORT :strings "\nNo ancillary detector in use"
  } else {
    :modify REPORT :strings "\nAn ancillary detector is in use\n>$_variable(detector)"
  }
#    page 7
  :modify REPORT :strings "\n\n5)Filenames of data saved as histograms which demonstrate the fault.\n> $_variable(hists)"
#    page 8
  if {$_variable(repeatable) == $_variable(emptylist)} then {
     :modify REPORT :strings "\n\n6)Can the fault be easily re-created?  - No"
  } else {
     :modify REPORT :strings "\n\n6)Can the fault be easily re-created?  - Yes\n> $_variable(repeatable)"
  }
#    page 9
     :modify REPORT :strings "\n\n7)Severity of the fault\n> $_variable(severity)"

     :modify REPORT :strings "\n\nEnd of report"
     :modify REPORT :see 0
}



proc start args {
   global _variable currentpage
    set currentpage 0
    set _variable(description) ""
    set _variable(detector) ""
    set _variable(hists) ""
    set _variable(repeatable) ""
    set _variable(severity) ""
    goto_nextpage
}

proc complete args {
   global directory _variable maillist
   get_text REPORT
   set z [catch {:file mkdir $directory} m]
   set reportfile $directory/[date +%d-%h-%y-%T]
   set-footer "saving report in file $reportfile"
   set z [catch {open $reportfile w} file]
   if {$z != 0} then {set-footer $file; return}
   puts $file $_variable(report)
   close $file
   set-footer "sending report by email to $maillist"
   set z [catch {exec mail -tw $maillist < $reportfile} m]
   if {$z != 0} then {set-footer $m; return}
   :modify [self frame-name] :delete T
}

proc cancel args {:modify [self frame-name] :delete T}
