
#    testcard.tcl    control for testing VXI module

set frame TESTCARD
set title "Test VXI Module Setup and Control"
set icontitle TestCard

set defaultchanlist {0 1 2 3 4 5 6 7}

set chanlist {???}
set chanindex 0

#    detector channels per VXI card
set nsockets 1

set module 0
set channel 0

set cardtype  Unknown
set cardtypewidget text
set VXIGlobal ""

set registerclasses [list xSht xDig xADC xAdr xDAC xDSP xDlW xExt xFIT xHst xLAR xLAT xLCA xMty xMux xPls xRng xSup xThr xTrigMK2LCA xbsr]
set registerclass 1

set registerbase ""
set vxislot -1

set registers {}
set currentregister ""
set registerindex -1
set registeraction 0
set registerdata ""

loadsource common.vxi
loadsource vxicontrol.common

#    Note - define acts on the whole module (VXI card)
proc click_define {} {
   global CurrentVXIServer cardtype channel module nsockets EGreport
   global vxislot

   loadsource [string tolower $cardtype].defn

   set CurrentVXIServer [VXImodule2crate $cardtype$channel]

   if {$nsockets == 1} then {
      set-footer "defining VXI module $cardtype$channel"
   } else {
      set-footer "defining VXI module $cardtype#$module"
   }
   set-busy
   set Save_EGreport $EGreport
   set EGreport -1
   if {$nsockets == 1} then {
      Add[set cardtype]Module $channel $vxislot
   } else {
      Add[set cardtype]Module $module $vxislot
   }
   set EGreport $Save_EGreport
   clear-busy
   clear-footer
}

#    Note - initialise acts on the whole module (VXI card)
proc click_initialise {} {
   global CurrentVXIServer cardtype channel module nsockets
   set CurrentVXIServer [VXImodule2crate $cardtype$channel]
   if {$nsockets == 1} then {
      initialise-vxi-registers  "$cardtype$channel.*"
   } else {
      for {set socket 1} {$socket <= $nsockets} {incr socket} {
         set c [expr $module * $nsockets + $socket - 1]
         initialise-vxi-registers  "$cardtype$c.*"
      }
   }
}


#    Note - reset, halt and go act on the whole module (VXI card)
proc click_reset {} {
   global CurrentVXIServer cardtype channel module
   set CurrentVXIServer [VXImodule2crate $cardtype$channel]
   if {[info procs [set cardtype]_DAQ-reset] != ""} then {[set cardtype]_DAQ-reset $cardtype$module} \
       else {set-footer "No [set cardtype]_DAQ-reset procedure"}
}

proc click_halt {} {
   global CurrentVXIServer cardtype channel module
   set CurrentVXIServer [VXImodule2crate $cardtype$channel]
   if {[info procs [set cardtype]_DAQ-halt] != ""} then {[set cardtype]_DAQ-halt $cardtype$module} \
       else {set-footer "No [set cardtype]_DAQ-halt procedure"}
}

proc click_go {} {
   global CurrentVXIServer cardtype channel module
   set CurrentVXIServer [VXImodule2crate $cardtype$channel]
   if {[info procs [set cardtype]_DAQ-go] != ""} then {[set cardtype]_DAQ-go $cardtype$module} \
       else {set-footer "No [set cardtype]_DAQ-go procedure"}
}

proc click_setup {} {
   global CurrentVXIServer cardtype channel module
   set CurrentVXIServer [VXImodule2crate $cardtype$channel]
   if {[info procs [set cardtype]_DAQ-setup] != ""} then {[set cardtype]_DAQ-setup $cardtype$module} \
       else {set-footer "No [set cardtype]_DAQ-setup procedure"}
}

proc repaint-channel {} {
   global CurrentVXIServer
   global chanindex channel nsockets module cardtype cardtypewidget vxislot
   set CurrentVXIServer [VXImodule2crate $cardtype$channel]
   :modify CHANNEL :value $chanindex
   if {$nsockets > 1} then {
     set attributes [read-vxi-attribute [set cardtype]$channel.ModuleName]
     set module [string range [first [split [first $attributes] .]] [expr [string length $cardtype] + 3] end]
     :modify M1 :label $module
     set attributes [read-vxi-attribute [set cardtype]#$module.VXI.Passed]
     set vxislot [string range [first $attributes] 2 end]
   } else {
     set attributes [read-vxi-attribute [set cardtype]$channel.VXI.Passed]
     set vxislot [string range [first $attributes] 2 end]
   }
   :modify VXISLOT :value $vxislot
   if {$cardtypewidget == "text"} then {
      :modify CARDTYPE :value $cardtype
   } else {
      set modulenames [inform vxicontrol inquire modulenames]
      :modify CARDTYPE :value [lsearch $modulenames $cardtype]
   }
}

proc click_vxiregisters {} {
   global CurrentVXIServer cardtype channel
   set CurrentVXIServer [VXImodule2crate $cardtype$channel]
   set data [read-vxi-registers $cardtype$channel.VXI.*]
   set info ""
   foreach item $data {
       append info "[format "%30s %20s" [first $item] [second $item]]\n"
   }
   midas-information $info
}

proc click_updateregisters {} {
   global CurrentVXIServer cardtype channel
   global registers
   set CurrentVXIServer [VXImodule2crate $cardtype$channel]
   set data [read-vxi-attributes $cardtype$channel.*]
   set registers {}
   foreach item $data {
       lappend registers "[format "%30s %100s" [first $item] [second $item]]"
   }
   :modify REGL :append F :strings $registers :see 0
}

proc click_editregisters {} {
   global env registeraction registerclasses registerclass

   :frame [self frame-name].registers :label  "VXI Registers for [self frame-title]" :show-footer T  \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

     :panel REGP1 :layout T
       :list REGL :rows 30  :cols 132 :exclusive T :scroll B
     :panel REGP2 :layout B :padx 10 :pady 10
      :layout H; :col-gap 20; :row-gap 5
       :text REGREG  :label "selected register" :w 120 :valuex 20c :shifters T
      :next-row T
       :text REGDATA  :label "register value"    :w 120 :valuex 20c
      :next-row T
       :button REGNEW    :label "define register"  :command click_defineregister
       :button REGDELETE :label "delete register"  :command click_deleteregister
       :button REGMODIFY :label "modify register"  :command click_modifyregister
       :checkbox REGACTION :label "apply to module"   :value $registeraction
       :button REGUPDATE   :label "update list from module"  :command click_updateregisters
       :choice-stack REGCLASS  :label "define using class" :menu $registerclasses :value $registerclass
      :next-row T
       :button REGREAD   :label "read register"    :command click_readregister
       :button REGWRITE  :label "write register"   :command click_writeregister
       :button REGINIT   :label "initialise register"    :command click_initialiseregister
       :button REGSAVE   :label "save"  :inactive T
       :hskip 50
       :checkbox VXITRACEMODE :label "VXI crate TraceMode"

   :show T
   click_updateregisters
}

proc click_selectregister {v i} {
  global currentregister registerindex registerdata
   set currentregister "[first $v]   [tail $v]"
   set registerindex $i
   :modify REGREG :value $currentregister
   set registerdata ""
   :modify REGDATA :value $registerdata
}

proc click_editregister {v} {
  global currentregister
  set currentregister $v
}

proc click_deleteregister {} {
   global currentregister registerindex registerdata registers registeraction
   set registers [lreplace $registers $registerindex $registerindex]
   :modify REGL :append F :strings $registers :see 0
   if {$registeraction == 1} then {delete-vxi-register [first $currentregister]}
   set registerindex -1
   set currentregister ""
   :modify REGREG :value $currentregister
   set registerdata ""
   :modify REGDATA :value $registerdata
}

proc click_defineregister {} {
   global currentregister registerindex registerdata registers registeraction
   global registerclasses registerclass
   set r [format "%30s %100s" [first $currentregister] [tail $currentregister]]
   set registers [lappend registers $r]
   :modify REGL :append F :strings $registers
   if {$registeraction == 1} then {
      set r [split [first $currentregister] .]
      set mname [first $r]
      set rname [join [tail $r] .]
      set class [lindex $registerclasses $registerclass]
      set attributes [tail $currentregister]
      define-vxi-register $mname $class $rname $attributes
   }
   set registerindex [expr [llength $registers] -1]
   set registerdata ""
   :modify REGDATA :value $registerdata
}

proc click_modifyregister {} {
   global currentregister registerindex registerdata registers registeraction
   set r [format "%30s %100s" [first $currentregister] [tail $currentregister]]
   set registers [lreplace $registers $registerindex $registerindex $r]
   :modify REGL :append F :strings $registers :see $registerindex
   if {$registeraction == 1} then {write-vxi-attribute [first $currentregister] [tail $currentregister]}
   set registerdata ""
   :modify REGDATA :value $registerdata
}

proc click_readregister {} {
   global currentregister registerdata
   set registerdata [read-vxi-register [first $currentregister]]
   :modify REGDATA :value $registerdata
}

proc click_writeregister {} {
   global currentregister registerdata
   write-vxi-register [first $currentregister] $registerdata
   set registerdata [read-vxi-register [first $currentregister]]
   :modify REGDATA :value $registerdata
}

proc click_registerdata {v} {
   global  registerdata
   set registerdata $v
}
   
proc click_initialiseregister {} {
   global currentregister registerdata
   initialise-vxi-register [first $currentregister]
   set registerdata [read-vxi-register [first $currentregister]]
   :modify REGDATA :value $registerdata
}

proc click_registers {} {
   global CurrentVXIServer
   global cardtype channel
   set CurrentVXIServer [VXImodule2crate $cardtype$channel]
   set data [read-vxi-registers $cardtype$channel.*]
   set info ""
   foreach item $data {
       append info "[format "%30s %20s" [first $item] [second $item]]\n"
   }
   midas-information $info
}

proc click_build {} {
   global cardtype

   if {[midas-check-for-temp] == 0} then {return}
   set z [Build[set cardtype]Module]
   if {$z == 0} then {
       set-footer "file [file join [pwd] [string tolower $cardtype].defn.c] built"
   } else {
       midas-warning "Unable to create [file join [pwd] [string tolower $cardtype].defn.c]\n$z"
   }
}

proc click arg {
   global debug registeraction registerclass vxislot
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch -glob $w {

      CHANNEL     {click_channel $v}
      CARDTYPE    {click_cardtype $v}
      VXISLOT     {set vxislot $v}

      REGL        {click_selectregister $v $i}
      REGREG      {click_editregister $v}
      REGDATA     {click_registerdata $v}
      REGACTION   {set registeraction $v}
      REGCLASS    {set registerclass $v}

      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}

proc click_cardtype {v} {
   global cardtype 
   set modulenames [inform vxicontrol inquire modulenames]
   set cardtype [lindex $modulenames $v]
   loadsource [string tolower $cardtype].defn
   start
}

proc click_channel v {
   global cardtype chanindex chanlist channel CurrentVXIModule
   set chanindex $v
   set channel [lindex $chanlist $chanindex]
   set CurrentVXIModule $cardtype$channel
   make-myserver
   repaint-channel
#   :modify VXITRACEMODE :value [read-vxi-register VXI.TraceMode]
}

proc click-vxitracemode {v} {
    write-vxi-register VXI.TraceMode $v
    :modify VXITRACEMODE :value [read-vxi-register VXI.TraceMode]
}

proc make-frame {} {
  global env cardtype cardtypewidget nsockets vxislot

  :frame [self frame-name] :label "[self frame-title]" :show-footer T :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

   :panel P1 :layout T :padx 10 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :choice-stack CHANNEL :label "Current Channel:" :strings [list ???] :valuex 18c
      if {$cardtype != "Unknown"} then {
         :text CARDTYPE :label "Card Type" :w 12 :deaf T :value $cardtype
         set cardtypewidget  "text"
      } else {
         :choice-stack CARDTYPE :label "Card Type" :strings [list ???] :value 0
         set cardtypewidget  "choice"
         :hskip 80
      }
      :text VXISLOT :label "VXI slot" :w 3  :value $vxislot

     :next-row T
      :button B2  :label "VXI registers"       :command click_vxiregisters
      :button B3  :label "Edit registers"      :command click_editregisters
      :button B4  :label "Register values"     :command click_registers
      :button B5  :label "Build C code"        :command click_build

    if {$nsockets > 1} then {
      :next-row T
       :message M0 :label "VXI Module:" :bold T
       :message M1 :label "?" :bold T
    }

     :next-row T
      :button DEFINE :label "Define module registers"        :command click_define
      :button INIT   :label "Initialise  module registers"   :command click_initialise

     :next-row T
      :button RESET  :label "reset module"           :command click_reset
      :button SETUP  :label "setup module"           :command click_setup
      :button GO     :label "enable acquisition"     :command click_go
      :button HALT   :label "disable acquisition"    :command click_halt

 :show T
}

proc start args {
   global started cardtype CurrentVXIModule
   global frameformat allchans allsegs segindex
   global chanlist defaultchanlist chanindex channel module nsockets
   if {$started != 0} then {open-frame} else {
	set allchans 0
        if {$segindex >= 0} then {set allsegs 0}
        set frameformat 1
	make-frame
        set started 1
   }
   set-busy
   if {$cardtype != "Unknown"} then {
      make-chanlist
      set chanindex 0
      set channel [lindex $chanlist $chanindex]
      set CurrentVXIModule $cardtype$channel
      make-myserver
      repaint-channel
#      :modify VXITRACEMODE :value [read-vxi-register VXI.TraceMode]
   } else {
      set chanlist $defaultchanlist
      set chanindex 0
      :modify CHANNEL :strings $chanlist :value $chanindex
      :modify CARDTYPE :strings [inform vxicontrol inquire modulechoice] :value 0
      set channel [lindex $chanlist $chanindex]
      if {$nsockets > 1} then {
         set module [expr $c / $nsockets]
         :modify M1 :label $module
      } else {
         set module $channel
      }
   }

   clear-busy
}

