# triggermk2.tcl    Trigger MK2 Interface control

#  created 30-7-97  VP  from fvi

set debug 0
set started 0

set frame TrigMK2
set frametitle "Trigger MK2 Interface Setup"
set icontitle  "TrigMK2"

set cardtype TriggerMK2
set VXIGlobal ""

set IsExoGam 0

#    for load LCA directory browser
set browserdirectory [file join $env(MIDASAPPLBASE) lca TrigMK2lca]
set browserfile ""

#    default scaler readout mode is counts
set scalermode 0


#    The array "table" which is indexed by the "widget name" contains all the information
#    needed for the register/display item
#
#    Each element of "table" is a list
#
#       0.    name of global parameter used to hold current value of the register
#       1.    procedure to be used to convert the widget value into parameter value
#       2.    procedure to be used to convert the parameter value into a widget value
#       3.    name of the register
#       4.    procedure to be used to convert the register value into parameter value
#       5.    procedure to be used to convert the parameter value into a register value
#       6.    name of unit used for regsiter access
#       7.    widget type to be used for "by parameter" displays
#       8-10  parameters for the procedure creating "by parameter" displays
#              eg  if (7) is :slider then 8 is min value, 9 is max value and 10 is unit
#       11    for startup only - initial value of the parameter (1)
#       12    for startup only - inserted for this widget into the table "parameternames"

#  Output Pulse timing

set table(FTDELAY) {ftdelay slider-widget->param param->int-widget "FT.Delay"
       unit-reg->param param->unit-reg ns nil nil nil nil 0 nil}
set table(FTWIDTH) {ftwidth slider-widget->param param->int-widget "FT.Width"
       unit-reg->param param->unit-reg ns nil nil nil nil 0 nil}
set table(INHIBITDELAY) {inhibitdelay slider-widget->param param->int-widget "Inhibit.Delay"
       unit-reg->param param->unit-reg ns nil nil nil nil 0 nil}
set table(INHIBITWIDTH) {inhibitwidth slider-widget->param param->int-widget "Inhibit.Width"
       unit-reg->param param->unit-reg ns nil nil nil nil 0 nil}
set table(FTVALDELAY) {ftvaldelay slider-widget->param param->int-widget "FTVal.Delay"
       unit-reg->param param->unit-reg ns nil nil nil nil 0 nil}
set table(FTVALWIDTH) {ftvalwidth slider-widget->param param->int-widget "FTVal.Width"
       unit-reg->param param->unit-reg ns nil nil nil nil 0 nil}
set table(VALDELAY) {valdelay slider-widget->param param->int-widget "Val.Delay"
       unit-reg->param param->unit-reg ns nil nil nil nil 0 nil}
set table(VALWIDTH) {valwidth slider-widget->param param->int-widget "Val.Width"
       unit-reg->param param->unit-reg ns nil nil nil nil 0 nil}

#   scalers

foreach i {1 2 3 4 5 6 7 8} {
   set table(SCALER$i) "scaler$i int-widget->param param->int-widget \"Scaler$i\"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil"
   set table(SCALER[set i]ENABLE) "scaler[set i]enable int-widget->param param->int-widget \"Scaler$i.Enable\"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil"
   set table(SCALER[set i]OVERFLOW) "scaler[set i]overflow int-widget->param param->int-widget \"Scaler$i.Overflow\"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil"
}

#    VXI Configuration Registers

set table(MODELCODE) {vximodcode str-widget->param param->hex-widget "VXI.ModelCode"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(SERIALNUM) {vxiserialnum str-widget->param param->hex-widget "VXI.SerialNum"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(MODLEVEL) {vximodlevel str-widget->param param->hex-widget "VXI.ModLevel"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(MANID) {vximanid str-widget->param param->hex-widget "VXI.ManID"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(OFFSET) {vxioffset str-widget->param param->hex-widget "VXI.Offset"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(DEVICE) {vxidevice str-widget->param param->hex-widget "VXI.DevClass"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(ADDRSPACE) {vxiaddrspace str-widget->param param->hex-widget "VXI.AddrSpace"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(STATUS) {vxistatus str-widget->param param->hex-widget "VXI.Status"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(LASTCARD) {lastcard int-widget->param param->int-widget "LastCard"
       int-reg->param param->int-reg nil :checkbox-c nil nil nil 1 "Last Card"}

set table(RESET-FIFO) {resetfifo int-widget->param param->int-widget "FIFO.Reset" 
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(RESET-GIR) {resetgir int-widget->param param->int-widget "GIR.Reset" 
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(RESETFIFOS) {resetfifos int-widget->param param->int-widget "ResetFIFOs" 
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

#    System Control Register

set table(VXICLEAR) {vxiclear int-widget->param param->int-widget "VXI.Clear"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(VXISYNCTRIG) {vxisynctrig int-widget->param param->int-widget "VXI.Sync.Trig"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(VXIGOSTOP) {vxigostop int-widget->param param->int-widget "VXI.Go"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(OPMODE) {opmode int-widget->param param->int-widget "ParallelMode"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

#    System/Board Status Register
set table(SYSB0) {sysb0 int-widget->param param->int-widget "SysSR.b0"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(SYSB1) {sysb1 int-widget->param param->int-widget "SysSR.b1"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(SYSB2) {sysb2 int-widget->param param->int-widget "SysSR.b2"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(SYSB3) {sysb3 int-widget->param param->int-widget "LCAProgrammed"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(SYSB4) {sysb4 int-widget->param param->int-widget "SysSR.b4"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

#    Card Control Register #2

set table(TACENABLE) {tacenable int-widget->param param->int-widget "TACADC.Enable"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(TREFENABLE) {trefenable int-widget->param param->int-widget "TRef.Enable"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(TACSTART) {tacstart int-widget->param param->int-widget "TACStartSelect"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(TACSTOP) {tacstop int-widget->param param->int-widget "TACStopSelect"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(SUMBUSCOMP) {sumbuscomp int-widget->param param->int-widget "SumbusCompSelect"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}


foreach i {1 2 3 4} {

#    set table(SUMBUS[set i]) {sumbusn int-widget->param param->int-widget 
#       \"Sumbus[set i].Enable\"
#       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

    set table(SUMBUS[set i]-RD) "sumbusnrd int-widget->param param->int-widget 
       \"RdOut.Sumbus[set i].Enable\"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil"
    set table(SUMBUS[set i]-ADDR) "sumbusnaddr str-widget->param param->hex-str-widget \"Sumbus[set i].Addr\"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil"
}

set table(TAC-RD) {tacrd int-widget->param param->int-widget  "RdOut.TAC.Enable"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(TAC-ADDR) {tacaddr str-widget->param param->hex-str-widget "TAC.Addr"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(PATT-RD) {pattrd int-widget->param param->int-widget  "RdOut.Pattern.Enable"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(PATT-ADDR) {pattaddr str-widget->param param->hex-str-widget "Pattern.Addr"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(EVNUMMSB-RD) {evnummsbrd int-widget->param param->int-widget  "RdOut.EvNumMSB.Enable"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(EVNUMMSB-ADDR) {evnummsbaddr str-widget->param param->hex-str-widget "EvNumMSB.Addr"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(EVNUMLSB-RD) {evnumlsbrd int-widget->param param->int-widget  "RdOut.EvNumLSB.Enable"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(EVNUMLSB-ADDR) {evnumlsbaddr str-widget->param param->hex-str-widget "EvNumLSB.Addr"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(LCAPROG)     {lcaprog          int-widget->param param->str-widget     "LCAFile"
                     reg->param param->reg         "" nil nil nil nil 0 nil}

#    Real Time Clock - ExoGam version only

set table(RTCW0-RD) {rtcw0rd int-widget->param param->int-widget  "RTC_W0.Enable"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(RTCW0-ADDR) {rtcw0addr str-widget->param param->hex-str-widget "RTC_W0.Addr"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(RTCW1-RD) {rtcw1rd int-widget->param param->int-widget  "RTC_W1.Enable"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(RTCW1-ADDR) {rtcw1addr str-widget->param param->hex-str-widget "RTC_W1.Addr"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(RTCW2-RD) {rtcw2rd int-widget->param param->int-widget  "RTC_W2.Enable"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(RTCW2-ADDR) {rtcw2addr str-widget->param param->hex-str-widget "RTC_W2.Addr"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(RTC-RUN) {rtcrun int-widget->param param->int-widget  "RTC_Run"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}


#    the following part of the table has a variable register name dependent on sumbus selected

proc update_table_sumbus {} {
  global sumbus table

  set table(SUMBUSN) "sumbusn int-widget->param param->int-widget 
       Sumbus[set sumbus].Enable
       int-reg->param param->int-reg nil nil nil nil nil 0 nil"

  foreach i {1 2 3 4} {
    set table(SUMBUSN[set i]DELAY)    "sumbusnndelay slider-widget->param param->int-widget
       \"Sumbus[set sumbus]#[set i].Delay\"
       unit-reg->param param->unit-reg ns nil nil nil nil 0 nil"
    set table(SUMBUSN[set i]WIDTH)    "sumbusnnwidth slider-widget->param param->int-widget 
       \"Sumbus[set sumbus]#[set i].Width\"
       unit-reg->param param->unit-reg ns nil nil nil nil 0 nil"
    set table(SUMBUSN[set i]THRESH)   "sumbusnnthresh int-widget->param real-param->int-widget 
       \"Sumbus[set sumbus]#[set i].Thresh\"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil"
  }
}

set sumbus 1
update_table_sumbus

#    the following part of the table has a variable register name dependent on logic input selected

proc update_table_logicinput {} {
  global logicinput table

    set table(LOGICINPUTNDELAY)    "logicinputndelay slider-widget->param param->int-widget 
       \"Logic[set logicinput].Delay\"
       unit-reg->param param->unit-reg ns nil nil nil nil 0 nil"
    set table(LOGICINPUTNWIDTH)    "logicinputnwidth slider-widget->param param->int-widget 
       \"Logic[set logicinput].Width\"
       unit-reg->param param->unit-reg ns nil nil nil nil 0 nil"
    set table(LOGICINPUTNTHRESH)   "logicinputnthresh slider-widget->param param->int-widget 
       \"Logic[set logicinput].Thresh\"
       unit-reg->param param->unit-reg mV nil nil nil nil 0 nil"

  set table(LOGICINPUTENABLE) "logicinputenable int-widget->param param->int-widget 
       LogicInput.Enable
       int-reg->param param->int-reg nil nil nil nil nil 0 nil"
}

set logicinput 1
update_table_logicinput


#    register selection for parameter save function

set registersavepatterns {
   FT.Delay FT.Width Inhibit.Delay Inhibit.Width FTVal.Delay FTVal.Width Val.Delay Val.Width 
   RdOut.Sumbus[1-4].Enable RdOut.TAC.Enable RdOut.Pattern.Enable RdOut.EvNumMSB.Enable RdOut.EvNumLSB.Enable
   Sumbus1#1.Delay Sumbus1#1.Width Sumbus1#1.Thresh
   Sumbus1#2.Delay Sumbus1#2.Width Sumbus1#2.Thresh
   Sumbus1#3.Delay Sumbus1#3.Width Sumbus1#3.Thresh
   Sumbus1#4.Delay Sumbus1#4.Width Sumbus1#4.Thresh
   Sumbus2#1.Delay Sumbus2#1.Width Sumbus2#1.Thresh
   Sumbus2#2.Delay Sumbus2#2.Width Sumbus2#2.Thresh
   Sumbus2#3.Delay Sumbus2#3.Width Sumbus2#3.Thresh
   Sumbus3#1.Delay Sumbus3#1.Width Sumbus3#1.Thresh
   Sumbus3#2.Delay Sumbus3#2.Width Sumbus3#2.Thresh
   Sumbus4#1.Delay Sumbus4#1.Width Sumbus4#1.Thresh
   Logic[1-4].Delay Logic[1-4].Width Logic[1-8].Thresh LogicInput.Enable
   ValSelect TACADC.Enable TACStartSelect TACStopSelect TRef.Enable SumbusCompSelect
   ParallelMode RdoutCompSrc
   CCR1 ScCR
   GIR.ROCR GIR.ValAck.TimeOut LCAFile
}


# inspection tables must be in same order as entries in the :choice-stacks
# this is achieved by using these tables when generating the choices
# index number returned maps directly to the signal number to be written to 
# the Signal Select register

set insp_table(MUX-AI1) {"Insp.A1" AI}
set insp_table(MUX-AI2) {"Insp.A2" AI}
set insp_table(MUX-DI1) {"Insp.D1" DI}
set insp_table(MUX-DI2) {"Insp.D2" DI}
set insp_table(MUX-VI)  {"Insp.V"  VI}

set insp_menu(MUX-AI1) analogueinsptable
set insp_menu(MUX-AI2) analogueinsptable
set insp_menu(MUX-DI1) digitalinsptable
set insp_menu(MUX-DI2) digitalinsptable
set insp_menu(MUX-VI)  voltageinsptable

set inspection_widgetnames {MUX-AI1 MUX-AI2 MUX-DI1 MUX-DI2 MUX-VI}
set inspection_regnames    {Insp.A1 Insp.A2 Insp.D1 Insp.D2 Insp.V}

#    the inspection line tables are in parallel
#    for channel select * implies value determined by adc channel
#    for parameter select * implies disconnect

set analogueinsptable {Disconnected Sumbus#1 Sumbus#2 Sumbus#3 Sumbus#4 TAC}
set AI-ChanSelect {0 1 1 1 1 1}
set AI-Param      {* 0 1 2 3 4}

set digitalinsptable {Disconnected
        SB1-GD1 SB1-COMP1 SB1-GD2 SB1-COMP2 SB1-GD3 SB1-COMP3 SB1-GD4 SB1-COMP4
        SB2-GD1 SB2-COMP1 SB2-GD2 SB2-COMP2 SB2-GD3 SB2-COMP3
        SB3-GD1 SB3-COMP1 SB3-GD2 SB3-COMP2
        SB4-GD1 SB4-COMP1 
        FP1-GD  FP1-COMP FP2-GD  FP2-COMP FP3-GD  FP3-COMP FP4-GD  FP4-COMP FP5-COMP FP6-COMP FP7-COMP FP8-COMP
        Fast_Trigger VALGATE-GD Validation INHIBIT-GD TREF-INSP FTGD-TRIG-INSP FPSTART FPSTOP TAC-START 
        TAC-STOP FT-TYPE-D0 FT-TYPE-D1 FT-TYPE-D2 FT-TYPE-D3
        TAC-ADC-BUSY$ TAC-ADC-CONV$ SB-CLK SB-RESET
        FT-FIFO-WCLK FT-FIFO-RCLK TT-FIFO-WCLK TT-FIFO-RCLK FT-SB-FIFO-WCLK FT-SB-FIFO-RCLK
        Clear Reject Inhibit_Action INHIBIT_REQ_FP Inhibit_Request Readout Coding
        Renin_lca Data_ack_lca Skip_L_lca No_More_lca Ctr_bit_0_lca Ctr_bit_1_lca Ctr_bit_2_lca 
        Ctr_clock_lca Data_rdy_lca LastPass_lca Timeout_lca EF_skip_l_lca clk25MHz_lca
        state_bit_0_lca state_bit_1_lca state_bit_2_lca
        Fast_Trigger Inhibit Validation Veto OrValack OrDeadTime StartReadoutSIROP Renin(SIROP)
        SIROPDTready0 SIROPDTready1 OR8CFDDelay Inhibit_Card SIROPDTack_buf SIROPEmpty0 SIROPEmpty1
        Readout_Event_Fault Readout_Event_Match SIROPLPASS
        CSA*(FifoChipSelect_A) CLKA(FifoClock_A) ENA(FifoEnablePort_A)
        AS*(VME) DS(VME) R/W*(VME) ModRoco
        CardRenin* BLTACK(Card) BLTACK(Bus) OE_Ident*
        CSB*(FifoChipSelect_B) CLKB(FifoClock_B) ENB(FifoEnablePort_B) OR_Fifo(EmptyFlag)
        FDTACK DTACK*(VME) CardRenout SysClk LT_Reset* Reset_A* Reset_Fifo* Decode_Sel
        Strobe_Dec2 Dclk DCS* DDin Coding_Bus ExternalReset_LT Readout_Event_Fault+
}
set DI-ChanSelect {0
                   1 1 1 1 1 1 1 1 1 1
                   1 1 1 1 1 1 1 1 1 1
                   1 1 1 1 1 1 1 1 1 1
                   1 1 1 1 1 1 1 1 1 1
                   1 1 1 1 1 1 1 1 1 1
                   1 1 1 1 1 1 1 1 1 1
                   1 1 1
                   1 1 1 1 1 1 1 1
                   1 1 1 1 1 1 1 1
                   32 32 32 32 32 32 32 32 32 32 
                   32 32 32 32 32 32 32 32 32 32
                   32 32 32 32 32 32 32 32 32 32 
                   32 32 32 32 32 32 32 32 32 32 
                   32 32 32 32 32 32 32 32
}
set DI-Param      {* 
                   1 2 3 4 5 6 7 8 9 10
                   11 12 13 14 15 16 17 18 19 20
                   21 22 23 24 25 26 27 28 29 30
                   31 32 33 34 35 36 37 38 39 40
                   41 42 43 44 45 46 47 48 49 50
                   51 52 53 54 55 56 57 58 59 60
                   61 62 63
                   96 97 98 99 100 101 102 103
                   104 105 106 107 108 109 110 111
                    1 2 3 4 5 6 7 8 9 10
                   11 12 13 14 15 16 17 18 19 20
                   21 22 23 24 25 26 27 28 29 30
                   31 32 33 34 35 36 37 38 39 40
                   41 42 43 44 45 46 47 48
}

set voltageinsptable {Disconnected
        Vee-24 Vee-18 Vee-12 Vee-5.2 Vee-5-Analogue Vee-2 Ground
        Vcc+5-Analogue Vcc+5-Digital Vcc+12 Vcc+18 Vcc+20 Vcc+24
}
set VI-ChanSelect {0 32 32 32 32 32 32 32 32 32 32 32 32 32}
set VI-Param      {*  0  1  2  3  4  5  6  7  8  9 10 11 12}

set digitalinspmenu {Disconnected
        {"Input Signals"
        SB1-GD1 SB1-COMP1 SB1-GD2 SB1-COMP2 SB1-GD3 SB1-COMP3 SB1-GD4 SB1-COMP4
        SB2-GD1 SB2-COMP1 SB2-GD2 SB2-COMP2 SB2-GD3 SB2-COMP3
        SB3-GD1 SB3-COMP1 SB3-GD2 SB3-COMP2
        SB4-GD1 SB4-COMP1 
        FP1-GD  FP1-COMP FP2-GD  FP2-COMP FP3-GD  FP3-COMP FP4-GD  FP4-COMP FP5-COMP FP6-COMP FP7-COMP FP8-COMP}
        Fast_Trigger VALGATE-GD Validation INHIBIT-GD TREF-INSP FTGD-TRIG-INSP FPSTART FPSTOP TAC-START 
        TAC-STOP FT-TYPE-D0 FT-TYPE-D1 FT-TYPE-D2 FT-TYPE-D3
        TAC-ADC-BUSY$ TAC-ADC-CONV$ SB-CLK SB-RESET
        FT-FIFO-WCLK FT-FIFO-RCLK TT-FIFO-WCLK TT-FIFO-RCLK FT-SB-FIFO-WCLK FT-SB-FIFO-RCLK
        Clear Reject Inhibit_Action INHIBIT_REQ_FP Inhibit_Request Readout Coding
        {"Readout LCA"
        Renin_lca Data_ack_lca Skip_L_lca No_More_lca Ctr_bit_0_lca Ctr_bit_1_lca Ctr_bit_2_lca 
        Ctr_clock_lca Data_rdy_lca LastPass_lca Timeout_lca EF_skip_l_lca clk25MHz_lca
        state_bit_0_lca state_bit_1_lca state_bit_2_lca}
        {"GIR lines #1"
        Fast_Trigger Inhibit Validation Veto OrValack OrDeadTime StartReadoutSIROP Renin(SIROP)
        SIROPDTready0 SIROPDTready1 OR8CFDDelay Inhibit_Card SIROPDTack_buf SIROPEmpty0 SIROPEmpty1
        Readout_Event_Fault Readout_Event_Match SIROPLPASS}
        {"GIR lines #2" 
        CSA*(FifoChipSelect_A) CLKA(FifoClock_A) ENA(FifoEnablePort_A)
        AS*(VME) DS(VME) R/W*(VME) ModRoco
        CardRenin* BLTACK(Card) BLTACK(Bus) OE_Ident*
        CSB*(FifoChipSelect_B) CLKB(FifoClock_B) ENB(FifoEnablePort_B) OR_Fifo(EmptyFlag)
        FDTACK DTACK*(VME) CardRenout SysClk LT_Reset* Reset_A* Reset_Fifo* Decode_Sel
        Strobe_Dec2 Dclk DCS* DDin Coding_Bus ExternalReset_LT Readout_Event_Fault+}
}

loadsource gir
loadsource  common.vxi

#proc EG {args} {puts stdout "EG: ($args)" }

proc setregnames {} {
  global regnames table

  update_table_sumbus
  update_table_logicinput

  set regnames {}
  set names [lsort [array names table]]
  foreach w $names {
     set line $table($w)
     if {[lindex $line 12] != "nil"} then {
        lappend regnames [lindex $line 3]
     }
  }
  foreach w $names {
     set line $table($w)
     if {[lindex $line 12] == "nil"} then {
        lappend regnames [lindex $line 3]
     }
  }
}

#    execute immediately to ensure registers start up complete

setregnames

#    procedures which select the sumbus or logic input must update the register table


proc click-sumbus v {
   global sumbus
   set sumbus $v
   setregnames
   switch $sumbus {
     1  {
        :modify SUMBUSN1DELAY  :inactive F
        :modify SUMBUSN1WIDTH  :inactive F
        :modify SUMBUSN1THRESH :inactive F
        :modify SUMBUSN2DELAY  :inactive F
        :modify SUMBUSN2WIDTH  :inactive F
        :modify SUMBUSN2THRESH :inactive F
        :modify SUMBUSN3DELAY  :inactive F
        :modify SUMBUSN3WIDTH  :inactive F
        :modify SUMBUSN3THRESH :inactive F
        :modify SUMBUSN4DELAY  :inactive F
        :modify SUMBUSN4WIDTH  :inactive F
        :modify SUMBUSN4THRESH :inactive F
     }
     2  {
        :modify SUMBUSN1DELAY  :inactive F
        :modify SUMBUSN1WIDTH  :inactive F
        :modify SUMBUSN1THRESH :inactive F
        :modify SUMBUSN2DELAY  :inactive F
        :modify SUMBUSN2WIDTH  :inactive F
        :modify SUMBUSN2THRESH :inactive F
        :modify SUMBUSN3DELAY  :inactive F
        :modify SUMBUSN3WIDTH  :inactive F
        :modify SUMBUSN3THRESH :inactive F
        :modify SUMBUSN4DELAY  :inactive T
        :modify SUMBUSN4WIDTH  :inactive T
        :modify SUMBUSN4THRESH :inactive T
     }
     3  {
        :modify SUMBUSN1DELAY  :inactive F
        :modify SUMBUSN1WIDTH  :inactive F
        :modify SUMBUSN1THRESH :inactive F
        :modify SUMBUSN2DELAY  :inactive F
        :modify SUMBUSN2WIDTH  :inactive F
        :modify SUMBUSN2THRESH :inactive F
        :modify SUMBUSN3DELAY  :inactive T
        :modify SUMBUSN3WIDTH  :inactive T
        :modify SUMBUSN3THRESH :inactive T
        :modify SUMBUSN4DELAY  :inactive T
        :modify SUMBUSN4WIDTH  :inactive T
        :modify SUMBUSN4THRESH :inactive T
      }
     4  {
        :modify SUMBUSN1DELAY  :inactive F
        :modify SUMBUSN1WIDTH  :inactive F
        :modify SUMBUSN1THRESH :inactive F
        :modify SUMBUSN2DELAY  :inactive T
        :modify SUMBUSN2WIDTH  :inactive T
        :modify SUMBUSN2THRESH :inactive T
        :modify SUMBUSN3DELAY  :inactive T
        :modify SUMBUSN3WIDTH  :inactive T
        :modify SUMBUSN3THRESH :inactive T
        :modify SUMBUSN4DELAY  :inactive T
        :modify SUMBUSN4WIDTH  :inactive T
        :modify SUMBUSN4THRESH :inactive T
      }
   }

}

proc click-logicinput v {
   global logicinput
   set logicinput $v
   setregnames
   if {$logicinput <= 4} then {
      :modify LOGICINPUTNDELAY :inactive F
      :modify LOGICINPUTNWIDTH :inactive F
   } else {
      :modify LOGICINPUTNDELAY :inactive T
      :modify LOGICINPUTNWIDTH :inactive T
   }
}

proc click-read-scalers {} {
  global scalermode
  if {$scalermode == 0} then {
#    counts
    write-vxi-reg XferScalers 0
    write-vxi-reg XferScalers 1
    update-pairs [read-vxi-regs Scaler?]
  } else {
#   rates
    write-vxi-reg XferScalers 0
    write-vxi-reg XferScalers 1
    set scalers1 [read-vxi-regs Scaler?]
    set-footer "Waiting 5 seconds to accumuluate Scalers count rate"
    set-busy
    after 5000
    clear-busy
    clear-footer
    write-vxi-reg XferScalers 0
    write-vxi-reg XferScalers 1
    set scalers2 [read-vxi-regs Scaler?]
    foreach i {0 1 2 3 4 5 6 7} {
       set count1 [second [lindex $scalers1 $i]]
       set count2 [second [lindex $scalers2 $i]]
       set rate [expr [expr $count2 - $count1] / 5]
       :modify SCALER[expr $i + 1] :value $rate
    }
  }
}

proc click-clear-scalers {} {
   write-vxi-reg ClearScalers 1
   write-vxi-reg ClearScalers 0
}

proc flush-FIFOs {r} {
   write-vxi-reg $r 0
   write-vxi-reg $r 1
   write-vxi-reg $r 0
}

#    to reset the Trigger MK2 via the CCR2 set bit 6 to 0 (and then unreset)
proc reset-board {} {
   write-vxi-reg NotResetBoard 0
   write-vxi-reg NotResetBoard 1
}

proc load_lca {} {
   upvar #0 browserdirectory directory
   upvar #0 browserfile file
   global EGallow
 
   clear-footer

   if {$file == ""} then {
       midas-warning "No LCA program file selected"
       return
   }

   set VXIstate [inform vxicontrol read-vxistate]
   if {$VXIstate != 0} then {
      if {[midas-confirmation "The VXI system is in the GO state\nDo you really wish to change the Trigger LCA program now?"] == 0} then {return}
   }

   set v [file join $directory $file]
   set-footer [self frame-name].browser "Loading LCA Program $v"
   set EGallow 0x2000a
   write-vxi-reg LCAFile [Midas-Map-Filename $v]
   set EGallow  0xffffffff
   set lcastate [read-vxi-reg LCAProgrammed]
   if {$lcastate == 1} then {
      destroy-frame [self frame-name].browser
      :modify LCAPROG :value [read-vxi-reg LCAFile]
      set-footer "LCA Program $v loaded"
   } else {
      :modify LCAPROG :value "Warning - LCA not correctly programmed"
      set-footer "LCA Program $v failed to load correctly"
      midas-warning "LCA Program $v failed to load correctly\nThis MUST be corrected before the Trigger will be useable"
   }
}

#    the following procedure is called during redisplay
proc repaint-global {} {
    set state [read-vxi-reg LCAProgrammed]
    if {$state == 0} then {
        :modify LCAPROG :value "Warning - LCA not correctly programmed"
        midas-warning "LCA Program not correctly loaded\nThis MUST be corrected before the Trigger will be useable"
    } else {
        :modify LCAPROG :value [read-vxi-reg LCAFile]
    }
    redisplay-sumbusLUT
}


proc click arg {
   global debug scalermode
   global lut_table lut_threshold luttableitem
   if {$debug} then {puts stderr "proc click called with argument \{$arg\}"}
#   insert-debug "proc click called with argument \{$arg\}"
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch -glob $w {
      *REDISPLAY  {click-redisplay}
      ENGINEERING  {click-$v}

      GIR*         {click-GIRwindow $w $v}

      LUTITEM*     {set lut_table([string range $w 7 end]) $v}
      LUTTHRESHOLD {set lut_threshold $v}
      LUTTABLE     {set luttableitem $v; redisplay-sumbusLUT}

      LOGICINPUT-SELECT  {click-logicinput $i; click-redisplay}
      SUMBUS-SELECT      {click-sumbus $i; click-redisplay}

      MUX-*  {click-inspection $w [lindex $arg 4]}

      SC-READ    {click-read-scalers}
      SC-CLR     {click-clear-scalers; click-read-scalers}
      SC-DISPLAY {set scalermode $v; click-read-scalers}

      MCRFIFORESET    {flush-FIFOs FIFO.Reset}
      CCR2RESETBOARD  {reset-board}

      default   {click-on $w $v}
   }
}


proc start args {
   global frameformat allchans allsegs started
   global chanlist chanindex myserver resource
   global IsExoGam registersavepatterns

   set allchans 0
   set allsegs 0
   set frameformat 1
   set chanlist {}
   set chanindex -1
   set myserver [VXImodule2crate TriggerMK2]
   if {[string length $myserver] > 16} then {set myserver VXI}
   set resource $myserver

   if {$started != 0} then {open-frame} else {

        set m 0x940
        catch {set m [read-vxi-reg VXI.ModelCode]}
        if {$m == 0x941} then {set IsExoGam 1} else {set IsExoGam 0}

        if {$IsExoGam == 1} then {lappend registersavepatterns RTC_Run RTC_W1.Enable RTC_W2.Enable RTC_W3.Enable}

        make-frame1
        EGbindings
        set started 1
   }
   click-redisplay
}

set electronicsframe "[self frame-name].electronics"

proc make-frame1 {} {
  global env analogueinsptable digitalinspmenu voltageinsptable 
  global sumbuslut_length
  global IsExoGam

set col1 16c
set col2 6c


  :frame "[self frame-name]" :label "[self frame-title]" :show-footer T  :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

  :panel P1 :layout T :padx 20 :pady 5
   :layout H; :row-gap 5; :col-gap 20
     :choice-stack SUMBUS-SELECT :label "Sumbus:" :strings [list 1 2 3 4]  
     :checkbox SUMBUSN :label "Sumbus Enable:" 
     :choice-stack SUMBUSCOMP :label "Front Panel Sumbus 1 Discriminator Output Select"   \
         :strings [list "Discriminator 1" "Discriminator 2" "Discriminator 3" "Discriminator 4"]
     :col-gap 10

   :next-row T
     :message MS1 :label "Delay ns"       :bold T :valuex 20c
     :message MS2 :label "Width ns"       :bold T :valuex 28c
     :message MS3 :label "Threshold"      :bold T :valuex 28c
   :next-row T 
     :slider SUMBUSN1DELAY  :label "Disc 1:" :min 0 :max 2000 
     :slider SUMBUSN1WIDTH  :label ""        :min 0 :max 2000 
     :slider SUMBUSN1THRESH :label ""        :min 1 :max $sumbuslut_length :resolution 1 :value $sumbuslut_length
   :next-row T
     :slider SUMBUSN2DELAY  :label "Disc 2:"   :min 0 :max 2000 
     :slider SUMBUSN2WIDTH  :label ""          :min 0 :max 2000 
     :slider SUMBUSN2THRESH :label ""          :min 1 :max $sumbuslut_length :resolution 1 :value $sumbuslut_length
   :next-row T
     :slider SUMBUSN3DELAY  :label "Disc 3:" :min 0 :max 2000 
     :slider SUMBUSN3WIDTH  :label ""        :min 0 :max 2000
     :slider SUMBUSN3THRESH :label ""        :min 1 :max $sumbuslut_length :resolution 1 :value $sumbuslut_length
   :next-row T
     :slider SUMBUSN4DELAY  :label "Disc 4:"  :min 0 :max 2000 
     :slider SUMBUSN4WIDTH  :label ""         :min 0 :max 2000 
     :slider SUMBUSN4THRESH :label ""         :min 1 :max $sumbuslut_length :resolution 1 :value $sumbuslut_length

   :next-row T
     :choice-stack LOGICINPUT-SELECT :label "Logic Input:" :valuex 15c :strings [list 1 2 3 4 5 6 7 8]
     :hskip 20
     :checkbox LOGICINPUTENABLE :label "Logic Input Enable:" 
   :next-row T
     :message ML1 :label "Delay ns"       :bold T :valuex 20c
     :message ML2 :label "Width ns"       :bold T :valuex 28c
     :message ML3 :label "Threshold mV"   :bold T :valuex 28c
   :next-row T
     :slider LOGICINPUTNDELAY  :label "" :min 0 :max 12000 
     :slider LOGICINPUTNWIDTH  :label "" :min 0 :max 12000
     :slider LOGICINPUTNTHRESH :label "" :min -3000 :max +3000 

 :panel P2 :layout T :padx 5 :pady 5 
   :layout H; :row-gap 5
     :message M6 :label "Output Pulse Timing:" :bold T
     :message M7 :label "Delay ns"             :bold T :valuex 10c
     :message M8 :label "Width ns"             :bold T :valuex 30c
   :next-row T
     :slider FTDELAY :label "Fast Trigger:" :min 0 :max 2000 :valuex $col1
     :hskip 7
     :slider FTWIDTH :label ""              :min 0 :max 2000 :valuex $col2
   :next-row T
     :slider INHIBITDELAY :label "Inhibit:" :min 0 :max 12000  :valuex $col1
     :slider INHIBITWIDTH :label ""         :min 0 :max 35000 :valuex $col2
   :next-row T
     :slider FTVALDELAY :label "Validation Gate:" :min 0 :max 12000 :valuex $col1
     :slider FTVALWIDTH :label ""                 :min 0 :max 12000 :valuex $col2
   :next-row T
     :slider VALDELAY :label "Validation:" :min 0 :max 12000 :valuex $col1
     :slider VALWIDTH :label ""            :min 0 :max 12000 :valuex $col2

   :next-row T
    :message M14 :label "TAC Controls:" :bold T
     :checkbox TACENABLE  :label "TAC Enable:"  :valuex 25c
     :checkbox TREFENABLE :label "Tref Enable:" :valuex 25c

   :next-row T
    :choice-stack TACSTART :label "TAC Start Select"  :valuex 25c \
      :strings [list "Front Panel TAC Start" "Sumbus 1 discriminator 1" "Sumbus 2 discriminator 1" "Front Panel Logic 1"]
   :next-row T
    :choice-stack TACSTOP :label "TAC Stop Select"   :valuex 25c \
      :strings [list "Front Panel TAC Stop" "Sumbus 1 discriminator 1" "Sumbus 2 discriminator 1" "Front Panel Logic 1"]

  if {$IsExoGam == 1} then {
   :next-row T
    :message M15  :label "Real-Time Clock:" :bold T
     :checkbox RTC-RUN :label "Run:"  :valuex 5c
    :message M16  :label Readout:

     :checkbox RTCW2-RD :label "2:" 
     :text RTCW2-ADDR   :label "" :w 5

     :checkbox RTCW1-RD :label "1:" 
     :text RTCW1-ADDR   :label "" :w 5

     :checkbox RTCW0-RD :label "0:" 
     :text RTCW0-ADDR   :label "" :w 5
  }

   :next-row T
   :vskip 10
    :text LCAPROG :label "Current LCA Program:" :valuex 25c  :w 60

 :panel P5 :layout R :after P1
   :layout H; :row-gap 0; :col-gap 0
   :vskip 10
     :message M12 :label "Readout Parameters:" :bold T
   :next-row T
     :checkbox SUMBUS1-RD :label "Sumbus 1 Mult:" :valuex $col1
     :text SUMBUS1-ADDR   :label "" :w 5
   :next-row T
     :checkbox SUMBUS2-RD :label "Sumbus 2 Mult:" :valuex $col1
     :text SUMBUS2-ADDR   :label "" :w 5
   :next-row T
     :checkbox SUMBUS3-RD :label "Sumbus 3 Mult:" :valuex $col1
     :text SUMBUS3-ADDR   :label "" :w 5
   :next-row T
     :checkbox SUMBUS4-RD :label "Sumbus 4 Mult:" :valuex $col1
     :text SUMBUS4-ADDR   :label "" :w 5
   :next-row T
     :checkbox TAC-RD :label "TAC:" :valuex $col1
     :text TAC-ADDR   :label "" :w 5
   :next-row T
     :checkbox PATT-RD :label "Pattern:" :valuex $col1
     :text PATT-ADDR   :label "" :w 5
   :next-row T
     :checkbox EVNUMMSB-RD :label "EvNum MSB:" :valuex $col1
     :text EVNUMMSB-ADDR   :label "" :w 5
   :next-row T
     :checkbox EVNUMLSB-RD :label "EvNum LSB:" :valuex $col1
     :text EVNUMLSB-ADDR   :label "" :w 5

   :next-row T
     :vskip 25; :col-gap 10
     :message M9 :label "Scalers" :bold T :valuex 10c
     :choice SC-DISPLAY :strings [list "Counts" "Rates"]
   :next-row T
    :hskip 100
     :message   SCM   :label "Enable Overflow"
   :next-row T
     :layout H; :col-gap 0
  set scalerlabels [list Ftr1 Ftr2 Ftr3 Ftr4 Ft Val Rdout User]
  foreach i {1 2 3 4 5 6 7 8} {
     :text SCALER$i    :label "[lindex $scalerlabels [expr $i - 1]]" :w 12 :deaf T :valuex 5c
     :checkbox SCALER[set i]ENABLE  :label ""
     :checkbox SCALER[set i]OVERFLOW :label "" :deaf T
    :next-row T
  }

    :panel PANEL3 :layout T :padx 10 :pady 10
      :layout H; :col-gap 10; :row-gap 10
      :menubutton MUX-AI1 :label "Analogue Inspection 1:" :w 22 :menu $analogueinsptable
      :message MUX-AI1-USE   :label ""
      :next-row T
      :menubutton MUX-AI2 :label "Analogue Inspection 2:" :w 22 :menu $analogueinsptable
      :message MUX-AI2-USE   :label ""
      :next-row T
      :menubutton MUX-DI1 :label "Digital Inspection 1:"  :w 22 :menu $digitalinspmenu
      :message MUX-DI1-USE   :label ""
      :next-row T
      :menubutton MUX-DI2 :label "Digital Inspection 2:"  :w 22 :menu $digitalinspmenu
      :message MUX-DI2-USE   :label ""
      :next-row T
      :menubutton MUX-VI  :label "Voltage Inspection:"    :w 22 :menu $voltageinsptable
      :message MUX-VI-USE   :label ""

   :panel PANEL6  :layout B :padx 10 :after FOOTER
    :layout H; :col-gap 10
      :button REDISPLAY 
      :menubutton ENGINEERING :label "Engineering"  :menu [list electronics software]
      GIRbutton
      :button LCA           :label "Load LCA"       :command lca-browser
      :button LUT           :label "Threshold LUTs" :command click-lut
      :hskip 15
      :button HELP          :command click-help
      :hskip 15
      :button SC-READ       :label "Update scalers"
      :button SC-CLR        :label "Clear scalers"
   :show T
}



proc make-electronics {} {
  global env electronicsframe electronicsvisible

  :frame "$electronicsframe" :label "Trigger MK2 Electronics" :show-footer T :resize F :transient T

    :panel ELECTRONICSPANEL :layout T :padx 5 :pady 5
     :layout V; :row-gap 5

      :message ELM1       :label "VXI Configuration Registers (G.I.R)" :bold T
      :text MANID         :label "Manufacturer ID:"    :w 6  :read-only T   :valuex 20c
      :text DEVICE        :label "Device Class:"       :w 6  :read-only T   :valuex 20c
      :text ADDRSPACE     :label "Address Space:"      :w 6  :read-only T   :valuex 20c
      :text MODELCODE     :label "Model Code:"         :w 6  :read-only T   :valuex 20c
      :text SERIALNUM     :label "Serial Number:"      :w 6  :read-only T   :valuex 20c
      :text MODLEVEL      :label "Modification Level:" :w 6  :read-only T   :valuex 20c
      :text STATUS        :label "Status Register"     :w 6  :read-only T   :valuex 20c
      :text OFFSET        :label "Offset Register"     :w 6  :read-only T   :valuex 20c
      :checkbox LASTCARD  :label "Last card"                 :deaf T   :valuex 20c
      :message ELM2       :label "System Control Register" :bold T
      :choice VXICLEAR    :label "VXI Clear Line" :strings [list Low High]  :valuex 30c
      :choice VXISYNCTRIG :label "VXI Trigger Sync Pulse Line" :strings [list Low High]  :valuex 30c
      :choice VXIGOSTOP   :label "VXI Stop/Go"  :strings [list Stop Go] :valuex 30c
      :choice OPMODE      :label "Operation Mode" :strings [list "Common Deadtime" "Parallel Mode"]
      :message ELM5       :label "System/Board Status Register" :bold T
      :checkbox SYSB0     :label "VXI GO Signal"  :bold T :valuex 30c :deaf T
      :checkbox SYSB1     :label "VXI Inhibit Request Signal"  :bold T :valuex 30c :deaf T
      :checkbox SYSB2     :label "VXI Inhibit Action Signal"  :bold T :valuex 30c :deaf T
      :checkbox SYSB3     :label "LCAs Programmed Status"  :bold T :valuex 30c :deaf T
      :checkbox SYSB4     :label "ADC FIFO Flags Status"  :bold T :valuex 30c :deaf T
      :message ELM3       :label "Module Control Register" :bold T 
      :button MCRFIFORESET  :label "Reset GIR FIFOs"
      :message ELM4       :label "Card Control Register #2" :bold T
      :button CCR2RESETBOARD :label "Reset Trigger FIFOs and Readout"
  
      :show T
   set electronicsvisible 1
}        

proc lca-browser {}  {
   upvar #0 browserdirectory directory
   upvar #0 browserfile file

   set frame [self frame-name].browser
   :frame $frame :label "Trigger MK2 LCA Program Selection" :show-footer T :transient T :resize F

   :panel BROWSER :padx 10 :pady 10 :col-gap 10 :row-gap 10
   :layout H
   :text DIRECTORY  :w 60 :shifters T  :value "$directory"  :command browser-directory
   :button BROWSER-UP :label Up :command browser-up
   :next-row T
   :list DIRECTORYLIST :exclusive T :rows 10 :cols 80  :command browser-selection
   :next-row T
   :text BROWSER-FILE :label File :w 60 :value "$file" :command browser-file
   :hskip 50
   :button BROWSER-OK :label "Load LCA Program" :command load_lca

   :show T

    browser-redisplay
}



#    procedures to select and set the Threshold Look Up Tables

set luttablevisible 0
set sumbuslut_length 40
for {set i 1} {$i <= $sumbuslut_length} {incr i} {set lut_table($i) 0}
set luttableitem 0
set lutmenu {}
set lut_threshold 0

#    Built-in Sumbus Threshold look-up-tables

set sumbustablemenu [list      \
    "4ma"      \
    "2ma"      \
    "1ma"      \
    "Sbg 1"    \
    "Sbg 2"    \
    "Sbg 3"    \
    "Exo 2"    \
]

set sumbusthreshLUT [list      \
    "6,17,28,40,51,64,75,86,99,110,122,133,145,156,167,178,189,200,211,222,233,244,255"    \
    "3,8,14,20,25,32,37,43,50,55,61,66,72,78,83,88,94,100,105,111,116,122,127,133,138,144,149,155,160,255"    \
    "2,4,7,10,12,16,18,21,25,27,31,33,36,39,41,44,47,50,52,55,58,61,64,67,69,72,74,78,80,83,86,88,90,255"    \
    "6,17,28,40,51,64,75,86,99,110,122,133,145,156,167,178,189,200,211,222,233,244,255"    \
    "5,18,31,44,57,70,83,96,109,122,135,148,161,174,187,200,213,250,250,250,250,250,250,250,250,250,250,250,250,250,255"    \
    "4,9,15,21,27,32,38,44,50,55,61,67,73,79,84,90,96,102,108,113,119,125,130,136,141,147,154,160,166,172,178,184,190,255"  \
    "2,6,9,13,17,21,25,29,33,55,61,66,72,78,83,88,94,100,105,111,116,122,127,133,138,144,149,155,160,255"    \
]


proc click-lut {} {
   global luttablevisible sumbuslut_length sumbustablemenu

    set frame [self frame-name].luttable
    :frame $frame :label "Trigger Sumbus Threshold LUT Selection" :show-footer T :transient T :resize F

   :panel LUTSELECTION :padx 10 :pady 10 :col-gap 10 :row-gap 10
    :layout H
   :choice-stack LUTTABLE :label "Sumbus Threshold LUT" 
    :next-row T
   :text LUTTHRESHOLD :label "Threshold" :w 3
    :next-row T

    set count 0
    for {set i 1} {$i <= $sumbuslut_length} {incr i} {
        :number LUTITEM$i  :label "$i" :value 0 :valuex 5c :w 4 :min 0 :max 255
        incr count
        if {$count == 4} then {:next-row T; set count 0}
    }

    :choice LUTDEFAULT :label "Select built-in LUT" :strings $sumbustablemenu :command click_sumbusdefaultLUT :value -1
    :next-row T
    :button LUTWRITE :label "Write Threshold & LUT to Trigger Module" :command write-sumbusLUT

    :show T
   set luttablevisible 1

   redisplay-sumbusLUT
}

proc redisplay-sumbusLUT {} {
    global luttablevisible lutmenu luttableitem sumbuslut_length lut_table lut_threshold

    if {$luttablevisible == 0} then {return}

#    obtain Sumbus Threshold register information

    set data [read-vxi-regs Sumbus?#?.Thresh]

#    build menu of thresholds

    set lutmenu {}
    foreach item $data {
        set regname [first $item]
        set name [second [split $regname .]]
        set sumbus [string range $name 6 6]
        set desc [string range $name 8 8]
        lappend lutmenu  "Sumbus $sumbus descriminator $desc"
    }
    :modify LUTTABLE :strings $lutmenu :value $luttableitem

#    show threshold for selected descriminator

    set lutdata [second [lindex $data $luttableitem]]
    set lut_threshold [first $lutdata]
    :modify LUTTHRESHOLD :value $lut_threshold

#    show table for selected descriminator

    for {set i 1} {$i <= $sumbuslut_length} {incr i} {set lut_table($i) 0}

    set lutdata [split [string range [second $lutdata] 2 end] ,]

    set i 1
    foreach item $lutdata {
        set lut_table($i) $item
        incr i
    }

    for {set i 1} {$i <= $sumbuslut_length} {incr i} {:modify LUTITEM$i :value $lut_table($i)}
}

proc write-sumbusLUT {} {
    global lutmenu luttableitem sumbuslut_length lut_table lut_threshold

    set data "$lut_threshold"
    append data " -t $lut_table(1)"
    for {set i 2} {$i <= $sumbuslut_length} {incr i} {
        if {$lut_table($i) == 0} then {break}
        append data ",$lut_table($i)"
    }

    set item [lindex $lutmenu $luttableitem]
    write-vxi-reg Sumbus[second $item]#[fourth $item].Thresh $data

    click-redisplay
}

proc click_sumbusdefaultLUT {w v} {
    global sumbusthreshLUT sumbuslut_length lut_table lut_threshold

    set built_in [split [lindex $sumbusthreshLUT $v] ,]

    for {set i 1} {$i <= $sumbuslut_length} {incr i} {set lut_table($i) 0}
    set i 1
    foreach item $built_in {
        set lut_table($i) $item
        incr i
    }

    for {set i 1} {$i <= $sumbuslut_length} {incr i} {:modify LUTITEM$i :value $lut_table($i)}

    set lut_threshold [llength $built_in]
    :modify LUTTHRESHOLD :value $lut_threshold

    :modify LUTDEFAULT :value -1
}



#proc make-chanlist {} {}
#proc make-myserver {} {}
#proc click-redisplay {} {}
