
#    vxicontrol.common.tcl

if {[info exists CurrentVXIServer] == 0} then {set CurrentVXIServer "VXI"}
if {[info exists CurrentVXIModule] == 0} then {set CurrentVXIModule ""}
if {[info exists LogFlag] == 0} then {set LogFlag 0}

if {[info exists VXIregister] == 0} then {set VXIregister ""}
if {[info exists TrigServer] == 0} then {set TrigServer "VXI"}
if {[info exists TrigModule] == 0} then {set TrigModule ""}

proc define-vxi-register {mname class rname attributes} {
   global CurrentVXIServer VXIregister
   global EGallow
   set VXIregister /$CurrentVXIServer/$mname.$rname
#   insert-debug "define-vxi-register $VXIregister $class $attributes"
#     until we fix the VxWorks regsiter server to made delete work fully
#     ignore an already exists return to create
   set egallow $EGallow
   set EGallow 0x20006
   EG "register :name '$VXIregister' :class '$class' !new"
   set EGallow $egallow
   EG "register :name '$VXIregister' :object '$attributes'"
}

proc delete-vxi-registers {registers} {
   global CurrentVXIServer VXIregister
   set VXIregister /$CurrentVXIServer/$registers
#   insert-debug "delete-vxi-registers $VXIregister"
   EG "register :pattern '$VXIregister' !delete"
}

proc delete-vxi-register {register} {
   global CurrentVXIServer VXIregister
   set VXIregister /$CurrentVXIServer/$register
   EG "register :name '$VXIregister' !delete"
}

proc write-vxi-attribute {register vl} {
   global CurrentVXIServer VXIregister
   set VXIregister /$CurrentVXIServer/$register
   EG  "register :name '$VXIregister' :object '$vl'"
}

proc modify-vxi-register {register vl} {
   global CurrentVXIServer VXIregister
   set VXIregister /$CurrentVXIServer/$register
   EG  "register :name '$VXIregister' :object '$vl'"
}

proc read-vxi-attribute {register} {
   global CurrentVXIServer VXIregister
   set VXIregister /$CurrentVXIServer/$register
   return  [EG "register :name '$VXIregister' ?object"]
}

proc write-vxi-attributes {register vl} {
   global CurrentVXIServer VXIregister
   set VXIregister /$CurrentVXIServer/$register
   EG  "register :pattern '$VXIregister' :object '$vl'"
}

proc modify-vxi-registers {register vl} {
   global CurrentVXIServer VXIregister
   set VXIregister /$CurrentVXIServer/$register
   EG  "register :pattern '$VXIregister' :object '$vl'"
}

proc read-vxi-attributes {register} {
   global CurrentVXIServer VXIregister
   set VXIregister /$CurrentVXIServer/$register
   return  [EG "register :pattern '$VXIregister' ?names-and-objects"]
}

proc initialise-vxi-register {register} {
   global CurrentVXIServer VXIregister
   set VXIregister /$CurrentVXIServer/$register
   EG  "register :name '$VXIregister' !initialise"
}

proc initialise-vxi-registers {registers} {
   global CurrentVXIServer VXIregister
   set VXIregister /$CurrentVXIServer/$registers
   EG  "register :pattern '$VXIregister' !initialise"
}

proc read-vxi-register {register} {
   global CurrentVXIServer TrigServer TrigModule VXIregister EGallow
   set EGallow 0x10002
   if {[first [split $register .]] == $TrigModule} then {
      set VXIregister /$TrigServer/$register
   } else {
      set VXIregister /$CurrentVXIServer/$register
   }
   EG "register :name '$VXIregister' ?value"
}

proc read-vxi-registers {register} {
   global CurrentVXIServer VXIregister EGallow
   set EGallow 0x10004
   set VXIregister /$CurrentVXIServer/$register
   EG "register :pattern '$VXIregister' ?names-and-values"
}

proc write-vxi-register {register value} {
   global CurrentVXIServer TrigServer TrigModule VXIregister EGallow
   set EGallow 0xffffffff
   if {[first [split $register .]] == $TrigModule} then {
      set VXIregister /$TrigServer/$register
   } else {
      set VXIregister /$CurrentVXIServer/$register
   }
   EG "register :name '$VXIregister' :value '$value'"
}

proc write-vxi-registers {register value} {
   global CurrentVXIServer VXIregister EGallow
   set EGallow 0xffffffff
   set VXIregister /$CurrentVXIServer/$register
   EG "register :pattern '$VXIregister' :value '$value'"
}

proc ersdefine {register class} {
   global CurrentVXIServer
   EG "register :name '/$CurrentVXIServer/$register' :class '$class' !new"
}

proc ersinquire {register} {
   global CurrentVXIServer
   EG "register :pattern '/$CurrentVXIServer/$register' ?names"
}

proc erswta {register args} {
   global CurrentVXIServer
   EG "register  :name '/$CurrentVXIServer/$register' :object '$args'"
}


#    proc to read 32 bits from module
#    address is VME address relative to module base
proc rdl {address} {return [rd l $address]}

#    proc to read 16 bits from module
#    address is VME address relative to module base
proc rdw {address} {return [rd w $address]}

#####################
proc rd {width address} {
   global CurrentVXIServer CurrentVXIModule LogFlag
   set a [expr $address]
   set register /$CurrentVXIServer/$CurrentVXIModule.data$width
   set attributes "-o [format "0x%04x" $a]"
   EG "register :name '$register' :object '$attributes'"
   set r [EG "register :name '$register' ?value"]
   if {$LogFlag == 1} then {log "$CurrentVXIModule: rd$width [format "0x%04x" $a]: $r"}
   return $r
}

#    proc to write 32 bits to module
#    address is VME address relative to module base
#####################
proc wrl {address args} {
   switch [llength $args] {
       0  {set r [wr l $address 0 0]}
       1  {set r [wr l $address [first $args] 0]}
       2  {set r [wr l $address [first $args] [second $args]]}
       default {midas-warning "wrong number of arguments in call to wrl -  $args"; return}
   }
}

#    proc to write 16 bits to module
#    address is VME address relative to module base
#####################
proc wrw {address args} {
   switch [llength $args] {
       0  {set r [wr w $address 0 0]}
       1  {set r [wr w $address [first $args] 0]}
       2  {set r [wr w $address [first $args] [second $args]]}
       default {midas-warning "wrong number of arguments in call to wrw -  $args"; return}
   }
}

#####################
proc wr {width address data flag} {
   global CurrentVXIServer CurrentVXIModule LogFlag
   set a [expr $address]
   set vl [expr $data]
   set register /$CurrentVXIServer/$CurrentVXIModule.data$width
   if {$LogFlag == 1} then {log "$CurrentVXIModule: wr$width [format "0x%04x" $a]: $vl"}
   set attributes "-o [format "0x%04x" $a]"
   EG "register :name '$register' :object '$attributes'"
   EG "register :name '$register' :value '$vl'"
   if {$flag == 0} then {return [EG "register :name '$register' ?value"]} else {return $data}
}

